# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.lxc as lxc
import salt.utils.versions

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class LxcTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.lxc
    """

    def setup_loader_modules(self):
        return {lxc: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to verify the named container if it exist.
        """
        name = "web01"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=[False, True, True, True, True, True, True])
        mock_t = MagicMock(
            side_effect=[
                None,
                True,
                "frozen",
                "frozen",
                "stopped",
                "running",
                "running",
            ]
        )
        with patch.dict(lxc.__salt__, {"lxc.exists": mock, "lxc.state": mock_t}):
            comt = "Clone source 'True' does not exist"
            ret.update({"comment": comt})
            self.assertDictEqual(lxc.present(name, clone_from=True), ret)

            with patch.dict(lxc.__opts__, {"test": True}):
                comt = "Container 'web01' will be cloned from True"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(lxc.present(name, clone_from=True), ret)

                comt = "Container 'web01' already exists"
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(lxc.present(name, clone_from=True), ret)

                comt = "Container 'web01' would be unfrozen"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    lxc.present(name, running=True, clone_from=True), ret
                )

                comt = "Container '{0}' would be stopped".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    lxc.present(name, running=False, clone_from=True), ret
                )

                comt = "Container 'web01' already exists and is stopped"
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(
                    lxc.present(name, running=False, clone_from=True), ret
                )

            with patch.dict(lxc.__opts__, {"test": False}):
                comt = "Container 'web01' already exists"
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(lxc.present(name, clone_from=True), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure a container is not present, destroying it if present.
        """
        name = "web01"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=[False, True, True])
        mock_des = MagicMock(return_value={"state": True})
        with patch.dict(lxc.__salt__, {"lxc.exists": mock, "lxc.destroy": mock_des}):
            comt = "Container '{0}' does not exist".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(lxc.absent(name), ret)

            with patch.dict(lxc.__opts__, {"test": True}):
                comt = "Container '{0}' would be destroyed".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(lxc.absent(name), ret)

            with patch.dict(lxc.__opts__, {"test": False}):
                comt = "Container '{0}' was destroyed".format(name)
                ret.update(
                    {"comment": comt, "result": True, "changes": {"state": True}}
                )
                self.assertDictEqual(lxc.absent(name), ret)

    # 'running' function tests: 1

    def test_running(self):
        """
        Test to ensure that a container is running.
        """
        name = "web01"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        mock = MagicMock(return_value={"state": {"new": "stop"}})
        mock_t = MagicMock(side_effect=[None, "running", "stopped", "start"])
        with patch.dict(
            lxc.__salt__, {"lxc.exists": mock, "lxc.state": mock_t, "lxc.start": mock}
        ):
            comt = "Container '{0}' does not exist".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(lxc.running(name), ret)

            comt = "Container 'web01' is already running"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(lxc.running(name), ret)

            with patch.dict(lxc.__opts__, {"test": True}):
                comt = "Container 'web01' would be started"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(lxc.running(name), ret)

            with patch.dict(lxc.__opts__, {"test": False}):
                comt = "Unable to start container 'web01'"
                ret.update(
                    {
                        "comment": comt,
                        "result": False,
                        "changes": {"state": {"new": "stop", "old": "start"}},
                    }
                )
                self.assertDictEqual(lxc.running(name), ret)

    # 'frozen' function tests: 1

    def test_frozen(self):
        """
        Test to ensure that a container is frozen.
        """
        name = "web01"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock = MagicMock(return_value={"state": {"new": "stop"}})
        mock_t = MagicMock(side_effect=["frozen", "stopped", "stopped"])
        with patch.dict(lxc.__salt__, {"lxc.freeze": mock, "lxc.state": mock_t}):
            comt = "Container '{0}' is already frozen".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(lxc.frozen(name), ret)

            with patch.dict(lxc.__opts__, {"test": True}):
                comt = "Container 'web01' would be started and frozen"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(lxc.frozen(name), ret)

            with patch.dict(lxc.__opts__, {"test": False}):
                comt = "Unable to start and freeze container 'web01'"
                ret.update(
                    {
                        "comment": comt,
                        "result": False,
                        "changes": {"state": {"new": "stop", "old": "stopped"}},
                    }
                )
                self.assertDictEqual(lxc.frozen(name), ret)

    # 'stopped' function tests: 1

    def test_stopped(self):
        """
        Test to ensure that a container is stopped.
        """
        name = "web01"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        mock = MagicMock(return_value={"state": {"new": "stop"}})
        mock_t = MagicMock(side_effect=[None, "stopped", "frozen", "frozen"])
        with patch.dict(lxc.__salt__, {"lxc.stop": mock, "lxc.state": mock_t}):
            comt = "Container '{0}' does not exist".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(lxc.stopped(name), ret)

            comt = "Container '{0}' is already stopped".format(name)
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(lxc.stopped(name), ret)

            with patch.dict(lxc.__opts__, {"test": True}):
                comt = "Container 'web01' would be stopped"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(lxc.stopped(name), ret)

            with patch.dict(lxc.__opts__, {"test": False}):
                comt = "Unable to stop container 'web01'"
                ret.update(
                    {
                        "comment": comt,
                        "result": False,
                        "changes": {"state": {"new": "stop", "old": "frozen"}},
                    }
                )
                self.assertDictEqual(lxc.stopped(name), ret)

    # 'set_pass' function tests: 1

    def test_set_pass(self):
        """
        Test to execute set_pass func.
        """
        comment = (
            "The lxc.set_pass state is no longer supported. Please see "
            "the LXC states documentation for further information."
        )
        ret = {"name": "web01", "comment": comment, "result": False, "changes": {}}

        self.assertDictEqual(lxc.set_pass("web01"), ret)

    # 'edited_conf' function tests: 1

    def test_edited_conf(self):
        """
        Test to edit LXC configuration options
        """
        name = "web01"

        comment = "{0} lxc.conf will be edited".format(name)

        ret = {"name": name, "result": True, "comment": comment, "changes": {}}

        with patch.object(salt.utils.versions, "warn_until", MagicMock()):
            with patch.dict(lxc.__opts__, {"test": True}):
                self.assertDictEqual(lxc.edited_conf(name), ret)

            with patch.dict(lxc.__opts__, {"test": False}):
                mock = MagicMock(return_value={})
                with patch.dict(lxc.__salt__, {"lxc.update_lxc_conf": mock}):
                    self.assertDictEqual(lxc.edited_conf(name), {"name": "web01"})



# -*- coding: utf-8 -*-
"""
    tests.integration.shell.auth
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

from __future__ import absolute_import, print_function, unicode_literals

import logging
import random
import string

import pytest
import salt.utils.platform
from salt.ext.six.moves import range
from salt.utils.pycrypto import gen_hash
from tests.support.case import ModuleCase, ShellCase
from tests.support.helpers import (
    destructiveTest,
    requires_salt_modules,
    requires_salt_states,
    skip_if_not_root,
    slowTest,
)
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.unit import skipIf

try:
    import pwd
    import grp
except ImportError:
    pwd, grp = None, None


log = logging.getLogger(__name__)


def gen_password():
    """
    generate a password and hash it
    """
    password = "".join(
        random.choice(string.ascii_letters + string.digits) for _ in range(20)
    )
    hashed_pwd = (
        password
        if salt.utils.platform.is_darwin()
        else gen_hash("salt", password, "sha512")
    )

    return password, hashed_pwd


@requires_salt_states("user.absent", "user.present")
@requires_salt_modules("shadow.set_password")
@skip_if_not_root
@skipIf(pwd is None or grp is None, "No pwd or grp module available")
@destructiveTest
@pytest.mark.windows_whitelisted
class UserAuthTest(ModuleCase, SaltReturnAssertsMixin, ShellCase):
    """
    Test user auth mechanisms
    """

    _call_binary_ = "salt"
    user = "saltdev"

    def setUp(self):
        ret = self.run_state("user.present", name=self.user, createhome=False)
        self.assertSaltTrueReturn(ret)

    def tearDown(self):
        ret = self.run_state("user.absent", name=self.user)
        self.assertSaltTrueReturn(ret)

    @slowTest
    def test_pam_auth_valid_user(self):
        """
        test that pam auth mechanism works with a valid user
        """
        password, hashed_pwd = gen_password()

        # set user password
        set_pw_cmd = "shadow.set_password {0} '{1}'".format(self.user, hashed_pwd)
        stdout, stderr, retcode = self.run_call(
            set_pw_cmd, catch_stderr=True, with_retcode=True
        )
        if stderr:
            log.warning(stderr)
        self.assertFalse(retcode, stderr)

        # test user auth against pam
        cmd = '-a pam "*" test.ping --username {0} --password {1}'.format(
            self.user, password
        )
        resp = self.run_salt(cmd)
        log.debug("resp = %s", resp)
        self.assertIn("minion", [r.strip(": ") for r in resp])

    @slowTest
    def test_pam_auth_invalid_user(self):
        """
        test pam auth mechanism errors for an invalid user
        """
        cmd = '-a pam "*" test.ping ' "--username nouser --password {0}".format(
            "abcd1234"
        )
        resp = self.run_salt(cmd)
        self.assertIn("Authentication error occurred", "".join(resp))


@requires_salt_states("group.absent", "group.present", "user.absent", "user.present")
@requires_salt_modules("shadow.set_password", "user.chgroups")
@skip_if_not_root
@skipIf(pwd is None or grp is None, "No crypt module available")
@destructiveTest
class GroupAuthTest(ModuleCase, SaltReturnAssertsMixin, ShellCase):
    """
    Test group auth mechanisms
    """

    _call_binary_ = "salt"

    user = "saltadm"
    group = "saltops"

    def setUp(self):
        ret = self.run_state("group.present", name=self.group)
        self.assertSaltTrueReturn(ret)
        ret = self.run_state(
            "user.present", name=self.user, createhome=False, groups=[self.group]
        )
        self.assertSaltTrueReturn(ret)
        stdout, stderr, retcode = self.run_call(
            "user.chgroups {0} {1} True".format(self.user, self.group),
            local=True,
            with_retcode=True,
            catch_stderr=True,
        )
        if stderr:
            log.warning(stderr)
        self.assertFalse(retcode, stderr)

    def tearDown(self):
        ret0 = self.run_state("user.absent", name=self.user)
        ret1 = self.run_state("group.absent", name=self.group)
        self.assertSaltTrueReturn(ret0)
        self.assertSaltTrueReturn(ret1)

    @slowTest
    def test_pam_auth_valid_group(self):
        """
        test that pam auth mechanism works for a valid group
        """
        password, hashed_pwd = gen_password()

        # set user password
        set_pw_cmd = "shadow.set_password {0} '{1}'".format(self.user, hashed_pwd)
        stdout, stderr, retcode = self.run_call(
            set_pw_cmd, catch_stderr=True, with_retcode=True
        )
        if stderr:
            log.warning(stderr)
        self.assertFalse(retcode, stderr)

        # test group auth against pam: saltadm is not configured in
        # external_auth, but saltops is and saldadm is a member of saltops
        cmd = '-a pam "*" test.ping --username {0} --password {1}'.format(
            self.user, password
        )
        resp = self.run_salt(cmd)
        self.assertIn("minion", [r.strip(": ") for r in resp])



# -*- coding: utf-8 -*-
"""
The generic libcloud template used to create the connections and deploy the
cloud virtual machines
"""
from __future__ import absolute_import, print_function, unicode_literals

import logging

# Import python libs
import os

import salt.client
import salt.config as config

# Import salt cloud libs
import salt.utils.cloud
import salt.utils.data

# Import salt libs
import salt.utils.event
from salt.exceptions import SaltCloudNotFound, SaltCloudSystemExit
from salt.ext import six
from salt.ext.six.moves import zip

# pylint: disable=W0611
# Import libcloud
try:
    import libcloud
    import re
    from libcloud.compute.types import Provider
    from libcloud.compute.providers import get_driver
    from libcloud.compute.deployment import MultiStepDeployment, ScriptDeployment

    HAS_LIBCLOUD = True
    LIBCLOUD_VERSION_INFO = tuple(
        [
            int(part)
            for part in libcloud.__version__.replace("-", ".")
            .replace("rc", ".")
            .split(".")[:3]
        ]
    )

except ImportError:
    HAS_LIBCLOUD = False
    LIBCLOUD_VERSION_INFO = (1000,)
# pylint: enable=W0611


# Get logging started
log = logging.getLogger(__name__)


LIBCLOUD_MINIMAL_VERSION = (0, 14, 0)


def node_state(id_):
    """
    Libcloud supported node states
    """
    states_int = {
        0: "RUNNING",
        1: "REBOOTING",
        2: "TERMINATED",
        3: "PENDING",
        4: "UNKNOWN",
        5: "STOPPED",
        6: "SUSPENDED",
        7: "ERROR",
        8: "PAUSED",
    }
    states_str = {
        "running": "RUNNING",
        "rebooting": "REBOOTING",
        "starting": "STARTING",
        "terminated": "TERMINATED",
        "pending": "PENDING",
        "unknown": "UNKNOWN",
        "stopping": "STOPPING",
        "stopped": "STOPPED",
        "suspended": "SUSPENDED",
        "error": "ERROR",
        "paused": "PAUSED",
        "reconfiguring": "RECONFIGURING",
    }
    return states_str[id_] if isinstance(id_, six.string_types) else states_int[id_]


def check_libcloud_version(reqver=LIBCLOUD_MINIMAL_VERSION, why=None):
    """
    Compare different libcloud versions
    """
    if not HAS_LIBCLOUD:
        return False

    if not isinstance(reqver, (list, tuple)):
        raise RuntimeError(
            "'reqver' needs to passed as a tuple or list, i.e., (0, 14, 0)"
        )
    try:
        import libcloud  # pylint: disable=redefined-outer-name
    except ImportError:
        raise ImportError(
            "salt-cloud requires >= libcloud {0} which is not installed".format(
                ".".join([six.text_type(num) for num in reqver])
            )
        )

    if LIBCLOUD_VERSION_INFO >= reqver:
        return libcloud.__version__

    errormsg = "Your version of libcloud is {0}. ".format(libcloud.__version__)
    errormsg += "salt-cloud requires >= libcloud {0}".format(
        ".".join([six.text_type(num) for num in reqver])
    )
    if why:
        errormsg += " for {0}".format(why)
    errormsg += ". Please upgrade."
    raise ImportError(errormsg)


def get_node(conn, name):
    """
    Return a libcloud node for the named VM
    """
    nodes = conn.list_nodes()
    for node in nodes:
        if node.name == name:
            __utils__["cloud.cache_node"](
                salt.utils.data.simple_types_filter(node.__dict__),
                __active_provider_name__,
                __opts__,
            )
            return node


def avail_locations(conn=None, call=None):
    """
    Return a dict of all available VM locations on the cloud provider with
    relevant data
    """
    if call == "action":
        raise SaltCloudSystemExit(
            "The avail_locations function must be called with "
            "-f or --function, or with the --list-locations option"
        )

    if not conn:
        conn = get_conn()  # pylint: disable=E0602

    locations = conn.list_locations()
    ret = {}
    for img in locations:
        if isinstance(img.name, six.string_types) and not six.PY3:
            img_name = img.name.encode("ascii", "salt-cloud-force-ascii")
        else:
            img_name = str(img.name)  # future lint: disable=blacklisted-function

        ret[img_name] = {}
        for attr in dir(img):
            if attr.startswith("_") or attr == "driver":
                continue

            attr_value = getattr(img, attr)
            if isinstance(attr_value, six.string_types) and not six.PY3:
                attr_value = attr_value.encode("ascii", "salt-cloud-force-ascii")
            ret[img_name][attr] = attr_value

    return ret


def avail_images(conn=None, call=None):
    """
    Return a dict of all available VM images on the cloud provider with
    relevant data
    """
    if call == "action":
        raise SaltCloudSystemExit(
            "The avail_images function must be called with "
            "-f or --function, or with the --list-images option"
        )

    if not conn:
        conn = get_conn()  # pylint: disable=E0602

    images = conn.list_images()
    ret = {}
    for img in images:
        if isinstance(img.name, six.string_types) and not six.PY3:
            img_name = img.name.encode("ascii", "salt-cloud-force-ascii")
        else:
            img_name = str(img.name)  # future lint: disable=blacklisted-function

        ret[img_name] = {}
        for attr in dir(img):
            if attr.startswith("_") or attr in ("driver", "get_uuid"):
                continue
            attr_value = getattr(img, attr)
            if isinstance(attr_value, six.string_types) and not six.PY3:
                attr_value = attr_value.encode("ascii", "salt-cloud-force-ascii")
            ret[img_name][attr] = attr_value
    return ret


def avail_sizes(conn=None, call=None):
    """
    Return a dict of all available VM images on the cloud provider with
    relevant data
    """
    if call == "action":
        raise SaltCloudSystemExit(
            "The avail_sizes function must be called with "
            "-f or --function, or with the --list-sizes option"
        )

    if not conn:
        conn = get_conn()  # pylint: disable=E0602

    sizes = conn.list_sizes()
    ret = {}
    for size in sizes:
        if isinstance(size.name, six.string_types) and not six.PY3:
            size_name = size.name.encode("ascii", "salt-cloud-force-ascii")
        else:
            size_name = str(size.name)  # future lint: disable=blacklisted-function

        ret[size_name] = {}
        for attr in dir(size):
            if attr.startswith("_") or attr in ("driver", "get_uuid"):
                continue

            try:
                attr_value = getattr(size, attr)
            except Exception:  # pylint: disable=broad-except
                pass

            if isinstance(attr_value, six.string_types) and not six.PY3:
                attr_value = attr_value.encode("ascii", "salt-cloud-force-ascii")
            ret[size_name][attr] = attr_value
    return ret


def get_location(conn, vm_):
    """
    Return the location object to use
    """
    locations = conn.list_locations()
    vm_location = config.get_cloud_config_value("location", vm_, __opts__)
    if not six.PY3:
        vm_location = vm_location.encode("ascii", "salt-cloud-force-ascii")

    for img in locations:
        if isinstance(img.id, six.string_types) and not six.PY3:
            img_id = img.id.encode("ascii", "salt-cloud-force-ascii")
        else:
            img_id = str(img.id)  # future lint: disable=blacklisted-function

        if isinstance(img.name, six.string_types) and not six.PY3:
            img_name = img.name.encode("ascii", "salt-cloud-force-ascii")
        else:
            img_name = str(img.name)  # future lint: disable=blacklisted-function

        if vm_location and vm_location in (img_id, img_name):
            return img

    raise SaltCloudNotFound(
        "The specified location, '{0}', could not be found.".format(vm_location)
    )


def get_image(conn, vm_):
    """
    Return the image object to use
    """
    images = conn.list_images()
    vm_image = config.get_cloud_config_value("image", vm_, __opts__)

    if not six.PY3:
        vm_image = vm_image.encode("ascii", "salt-cloud-force-ascii")

    for img in images:
        if isinstance(img.id, six.string_types) and not six.PY3:
            img_id = img.id.encode("ascii", "salt-cloud-force-ascii")
        else:
            img_id = str(img.id)  # future lint: disable=blacklisted-function

        if isinstance(img.name, six.string_types) and not six.PY3:
            img_name = img.name.encode("ascii", "salt-cloud-force-ascii")
        else:
            img_name = str(img.name)  # future lint: disable=blacklisted-function

        if vm_image and vm_image in (img_id, img_name):
            return img

    raise SaltCloudNotFound(
        "The specified image, '{0}', could not be found.".format(vm_image)
    )


def get_size(conn, vm_):
    """
    Return the VM's size object
    """
    sizes = conn.list_sizes()
    vm_size = config.get_cloud_config_value("size", vm_, __opts__)
    if not vm_size:
        return sizes[0]

    for size in sizes:
        if vm_size and str(vm_size) in (
            str(size.id),
            str(size.name),
        ):  # pylint: disable=blacklisted-function
            return size
    raise SaltCloudNotFound(
        "The specified size, '{0}', could not be found.".format(vm_size)
    )


def script(vm_):
    """
    Return the script deployment object
    """
    return ScriptDeployment(
        salt.utils.cloud.os_script(
            config.get_cloud_config_value("os", vm_, __opts__),
            vm_,
            __opts__,
            salt.utils.cloud.salt_config_to_yaml(
                salt.utils.cloud.minion_config(__opts__, vm_)
            ),
        )
    )


def destroy(name, conn=None, call=None):
    """
    Delete a single VM
    """
    if call == "function":
        raise SaltCloudSystemExit(
            "The destroy action must be called with -d, --destroy, " "-a or --action."
        )

    __utils__["cloud.fire_event"](
        "event",
        "destroying instance",
        "salt/cloud/{0}/destroying".format(name),
        args={"name": name},
        sock_dir=__opts__["sock_dir"],
        transport=__opts__["transport"],
    )

    if not conn:
        conn = get_conn()  # pylint: disable=E0602

    node = get_node(conn, name)
    profiles = get_configured_provider()["profiles"]  # pylint: disable=E0602
    if node is None:
        log.error("Unable to find the VM %s", name)
    profile = None
    if "metadata" in node.extra and "profile" in node.extra["metadata"]:
        profile = node.extra["metadata"]["profile"]

    flush_mine_on_destroy = False
    if profile and profile in profiles and "flush_mine_on_destroy" in profiles[profile]:
        flush_mine_on_destroy = profiles[profile]["flush_mine_on_destroy"]

    if flush_mine_on_destroy:
        log.info("Clearing Salt Mine: %s", name)

        mopts_ = salt.config.DEFAULT_MINION_OPTS
        conf_path = "/".join(__opts__["conf_file"].split("/")[:-1])
        mopts_.update(salt.config.minion_config(os.path.join(conf_path, "minion")))
        client = salt.client.get_local_client(mopts_)
        minions = client.cmd(name, "mine.flush")

    log.info("Clearing Salt Mine: %s, %s", name, flush_mine_on_destroy)
    log.info("Destroying VM: %s", name)
    ret = conn.destroy_node(node)
    if ret:
        log.info("Destroyed VM: %s", name)
        # Fire destroy action
        __utils__["cloud.fire_event"](
            "event",
            "destroyed instance",
            "salt/cloud/{0}/destroyed".format(name),
            args={"name": name},
            sock_dir=__opts__["sock_dir"],
            transport=__opts__["transport"],
        )
        if __opts__["delete_sshkeys"] is True:
            public_ips = getattr(node, __opts__.get("ssh_interface", "public_ips"))
            if public_ips:
                salt.utils.cloud.remove_sshkey(public_ips[0])

            private_ips = getattr(node, __opts__.get("ssh_interface", "private_ips"))
            if private_ips:
                salt.utils.cloud.remove_sshkey(private_ips[0])

        if __opts__.get("update_cachedir", False) is True:
            __utils__["cloud.delete_minion_cachedir"](
                name, __active_provider_name__.split(":")[0], __opts__
            )

        return True

    log.error("Failed to Destroy VM: %s", name)
    return False


def reboot(name, conn=None):
    """
    Reboot a single VM
    """
    if not conn:
        conn = get_conn()  # pylint: disable=E0602

    node = get_node(conn, name)
    if node is None:
        log.error("Unable to find the VM %s", name)
    log.info("Rebooting VM: %s", name)
    ret = conn.reboot_node(node)
    if ret:
        log.info("Rebooted VM: %s", name)
        # Fire reboot action
        __utils__["cloud.fire_event"](
            "event",
            "{0} has been rebooted".format(name),
            "salt-cloud" "salt/cloud/{0}/rebooting".format(name),
            args={"name": name},
            sock_dir=__opts__["sock_dir"],
            transport=__opts__["transport"],
        )
        return True

    log.error("Failed to reboot VM: %s", name)
    return False


def list_nodes(conn=None, call=None):
    """
    Return a list of the VMs that are on the provider
    """
    if call == "action":
        raise SaltCloudSystemExit(
            "The list_nodes function must be called with -f or --function."
        )

    if not conn:
        conn = get_conn()  # pylint: disable=E0602

    nodes = conn.list_nodes()
    ret = {}
    for node in nodes:
        ret[node.name] = {
            "id": node.id,
            "image": node.image,
            "name": node.name,
            "private_ips": node.private_ips,
            "public_ips": node.public_ips,
            "size": node.size,
            "state": node_state(node.state),
        }
    return ret


def list_nodes_full(conn=None, call=None):
    """
    Return a list of the VMs that are on the provider, with all fields
    """
    if call == "action":
        raise SaltCloudSystemExit(
            "The list_nodes_full function must be called with -f or --function."
        )

    if not conn:
        conn = get_conn()  # pylint: disable=E0602

    nodes = conn.list_nodes()
    ret = {}
    for node in nodes:
        pairs = {}
        for key, value in zip(node.__dict__, six.itervalues(node.__dict__)):
            pairs[key] = value
        ret[node.name] = pairs
        del ret[node.name]["driver"]

    __utils__["cloud.cache_node_list"](
        ret, __active_provider_name__.split(":")[0], __opts__
    )
    return ret


def list_nodes_select(conn=None, call=None):
    """
    Return a list of the VMs that are on the provider, with select fields
    """
    if not conn:
        conn = get_conn()  # pylint: disable=E0602

    return salt.utils.cloud.list_nodes_select(
        list_nodes_full(conn, "function"), __opts__["query.selection"], call,
    )


def show_instance(name, call=None):
    """
    Show the details from the provider concerning an instance
    """
    if call != "action":
        raise SaltCloudSystemExit(
            "The show_instance action must be called with -a or --action."
        )

    nodes = list_nodes_full()
    __utils__["cloud.cache_node"](nodes[name], __active_provider_name__, __opts__)
    return nodes[name]


def conn_has_method(conn, method_name):
    """
    Find if the provided connection object has a specific method
    """
    if method_name in dir(conn):
        return True

    log.error("Method '%s' not yet supported!", method_name)
    return False



# -*- coding: utf-8 -*-
"""
unit tests for the Salt engines
"""
from __future__ import absolute_import, print_function, unicode_literals

import logging

import salt.config
import salt.engines as engines
import salt.utils.process
from salt.ext import six
from tests.support.helpers import slowTest
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import patch
from tests.support.unit import TestCase

log = logging.getLogger(__name__)


class EngineTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.engine.sqs_events
    """

    def setup_loader_modules(self):
        return {engines: {}}

    @slowTest
    def test_engine_module(self):
        """
        Test
        """
        mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
        mock_opts["__role"] = "minion"
        mock_opts["engines"] = [
            {"test_one": {"engine_module": "test"}},
            {"test_two": {"engine_module": "test"}},
        ]

        process_manager = salt.utils.process.ProcessManager()
        with patch.dict(engines.__opts__, mock_opts):
            salt.engines.start_engines(mock_opts, process_manager)
            process_map = process_manager._process_map
            count = 0
            for proc in six.iterkeys(process_map):
                count += 1
                fun = process_map[proc]["Process"].fun

                # Ensure function is start from the test engine
                self.assertEqual(fun, "test.start")

            # Ensure there were two engine started
            self.assertEqual(count, len(mock_opts["engines"]))



# -*- coding: utf-8 -*-
"""
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.unit.utils.event_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""
from __future__ import absolute_import, print_function, unicode_literals

import hashlib
import os
import shutil
import time

import salt.config
import salt.ext.tornado.ioloop
import salt.utils.event
import salt.utils.stringutils
import zmq
import zmq.eventloop.ioloop
from salt.ext.six.moves import range
from salt.ext.tornado.testing import AsyncTestCase
from saltfactories.utils.processes.helpers import terminate_process
from tests.support.events import eventpublisher_process, eventsender_process
from tests.support.helpers import slowTest
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase, expectedFailure, skipIf

# support pyzmq 13.0.x, TODO: remove once we force people to 14.0.x
if not hasattr(zmq.eventloop.ioloop, "ZMQIOLoop"):
    zmq.eventloop.ioloop.ZMQIOLoop = zmq.eventloop.ioloop.IOLoop

NO_LONG_IPC = False
if getattr(zmq, "IPC_PATH_MAX_LEN", 103) <= 103:
    NO_LONG_IPC = True


@skipIf(
    NO_LONG_IPC, "This system does not support long IPC paths. Skipping event tests!"
)
class TestSaltEvent(TestCase):
    def setUp(self):
        self.sock_dir = os.path.join(RUNTIME_VARS.TMP, "test-socks")
        if not os.path.exists(self.sock_dir):
            os.makedirs(self.sock_dir)
        self.addCleanup(shutil.rmtree, self.sock_dir, ignore_errors=True)

    def assertGotEvent(self, evt, data, msg=None):
        self.assertIsNotNone(evt, msg)
        for key in data:
            self.assertIn(key, evt, "{0}: Key {1} missing".format(msg, key))
            assertMsg = "{0}: Key {1} value mismatch, {2} != {3}"
            assertMsg = assertMsg.format(msg, key, data[key], evt[key])
            self.assertEqual(data[key], evt[key], assertMsg)

    def test_master_event(self):
        me = salt.utils.event.MasterEvent(self.sock_dir, listen=False)
        self.assertEqual(
            me.puburi, "{0}".format(os.path.join(self.sock_dir, "master_event_pub.ipc"))
        )
        self.assertEqual(
            me.pulluri,
            "{0}".format(os.path.join(self.sock_dir, "master_event_pull.ipc")),
        )

    def test_minion_event(self):
        opts = dict(id="foo", sock_dir=self.sock_dir)
        id_hash = hashlib.sha256(
            salt.utils.stringutils.to_bytes(opts["id"])
        ).hexdigest()[:10]
        me = salt.utils.event.MinionEvent(opts, listen=False)
        self.assertEqual(
            me.puburi,
            "{0}".format(
                os.path.join(self.sock_dir, "minion_event_{0}_pub.ipc".format(id_hash))
            ),
        )
        self.assertEqual(
            me.pulluri,
            "{0}".format(
                os.path.join(self.sock_dir, "minion_event_{0}_pull.ipc".format(id_hash))
            ),
        )

    def test_minion_event_tcp_ipc_mode(self):
        opts = dict(id="foo", ipc_mode="tcp")
        me = salt.utils.event.MinionEvent(opts, listen=False)
        self.assertEqual(me.puburi, 4510)
        self.assertEqual(me.pulluri, 4511)

    def test_minion_event_no_id(self):
        me = salt.utils.event.MinionEvent(dict(sock_dir=self.sock_dir), listen=False)
        id_hash = hashlib.sha256(salt.utils.stringutils.to_bytes("")).hexdigest()[:10]
        self.assertEqual(
            me.puburi,
            "{0}".format(
                os.path.join(self.sock_dir, "minion_event_{0}_pub.ipc".format(id_hash))
            ),
        )
        self.assertEqual(
            me.pulluri,
            "{0}".format(
                os.path.join(self.sock_dir, "minion_event_{0}_pull.ipc".format(id_hash))
            ),
        )

    @slowTest
    def test_event_single(self):
        """Test a single event is received"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            me.fire_event({"data": "foo1"}, "evt1")
            evt1 = me.get_event(tag="evt1")
            self.assertGotEvent(evt1, {"data": "foo1"})

    @slowTest
    def test_event_single_no_block(self):
        """Test a single event is received, no block"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            start = time.time()
            finish = start + 5
            evt1 = me.get_event(wait=0, tag="evt1", no_block=True)
            # We should get None and way before the 5 seconds wait since it's
            # non-blocking, otherwise it would wait for an event which we
            # didn't even send
            self.assertIsNone(evt1, None)
            self.assertLess(start, finish)
            me.fire_event({"data": "foo1"}, "evt1")
            evt1 = me.get_event(wait=0, tag="evt1")
            self.assertGotEvent(evt1, {"data": "foo1"})

    @slowTest
    def test_event_single_wait_0_no_block_False(self):
        """Test a single event is received with wait=0 and no_block=False and doesn't spin the while loop"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            me.fire_event({"data": "foo1"}, "evt1")
            # This is too fast and will be None but assures we're not blocking
            evt1 = me.get_event(wait=0, tag="evt1", no_block=False)
            self.assertGotEvent(evt1, {"data": "foo1"})

    @slowTest
    def test_event_timeout(self):
        """Test no event is received if the timeout is reached"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            me.fire_event({"data": "foo1"}, "evt1")
            evt1 = me.get_event(tag="evt1")
            self.assertGotEvent(evt1, {"data": "foo1"})
            evt2 = me.get_event(tag="evt1")
            self.assertIsNone(evt2)

    @slowTest
    def test_event_no_timeout(self):
        """Test no wait timeout, we should block forever, until we get one """
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            with eventsender_process({"data": "foo2"}, "evt2", self.sock_dir, 5):
                evt = me.get_event(tag="evt2", wait=0, no_block=False)
            self.assertGotEvent(evt, {"data": "foo2"})

    @slowTest
    def test_event_matching(self):
        """Test a startswith match"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            me.fire_event({"data": "foo1"}, "evt1")
            evt1 = me.get_event(tag="ev")
            self.assertGotEvent(evt1, {"data": "foo1"})

    @slowTest
    def test_event_matching_regex(self):
        """Test a regex match"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            me.fire_event({"data": "foo1"}, "evt1")
            evt1 = me.get_event(tag="^ev", match_type="regex")
            self.assertGotEvent(evt1, {"data": "foo1"})

    @slowTest
    def test_event_matching_all(self):
        """Test an all match"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            me.fire_event({"data": "foo1"}, "evt1")
            evt1 = me.get_event(tag="")
            self.assertGotEvent(evt1, {"data": "foo1"})

    @slowTest
    def test_event_matching_all_when_tag_is_None(self):
        """Test event matching all when not passing a tag"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            me.fire_event({"data": "foo1"}, "evt1")
            evt1 = me.get_event()
            self.assertGotEvent(evt1, {"data": "foo1"})

    @slowTest
    def test_event_not_subscribed(self):
        """Test get_event drops non-subscribed events"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            me.fire_event({"data": "foo1"}, "evt1")
            me.fire_event({"data": "foo2"}, "evt2")
            evt2 = me.get_event(tag="evt2")
            evt1 = me.get_event(tag="evt1")
            self.assertGotEvent(evt2, {"data": "foo2"})
            self.assertIsNone(evt1)

    @slowTest
    def test_event_subscription_cache(self):
        """Test subscriptions cache a message until requested"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            me.subscribe("evt1")
            me.fire_event({"data": "foo1"}, "evt1")
            me.fire_event({"data": "foo2"}, "evt2")
            evt2 = me.get_event(tag="evt2")
            evt1 = me.get_event(tag="evt1")
            self.assertGotEvent(evt2, {"data": "foo2"})
            self.assertGotEvent(evt1, {"data": "foo1"})

    @slowTest
    def test_event_subscriptions_cache_regex(self):
        """Test regex subscriptions cache a message until requested"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            me.subscribe("e..1$", "regex")
            me.fire_event({"data": "foo1"}, "evt1")
            me.fire_event({"data": "foo2"}, "evt2")
            evt2 = me.get_event(tag="evt2")
            evt1 = me.get_event(tag="evt1")
            self.assertGotEvent(evt2, {"data": "foo2"})
            self.assertGotEvent(evt1, {"data": "foo1"})

    @slowTest
    def test_event_multiple_clients(self):
        """Test event is received by multiple clients"""
        with eventpublisher_process(self.sock_dir):
            me1 = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            me2 = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            # We need to sleep here to avoid a race condition wherein
            # the second socket may not be connected by the time the first socket
            # sends the event.
            time.sleep(0.5)
            me1.fire_event({"data": "foo1"}, "evt1")
            evt1 = me1.get_event(tag="evt1")
            self.assertGotEvent(evt1, {"data": "foo1"})
            evt2 = me2.get_event(tag="evt1")
            self.assertGotEvent(evt2, {"data": "foo1"})

    @expectedFailure
    def test_event_nested_sub_all(self):
        """Test nested event subscriptions do not drop events, get event for all tags"""
        # Show why not to call get_event(tag='')
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            me.fire_event({"data": "foo1"}, "evt1")
            me.fire_event({"data": "foo2"}, "evt2")
            evt2 = me.get_event(tag="")
            evt1 = me.get_event(tag="")
            self.assertGotEvent(evt2, {"data": "foo2"})
            self.assertGotEvent(evt1, {"data": "foo1"})

    @slowTest
    def test_event_many(self):
        """Test a large number of events, one at a time"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            for i in range(500):
                me.fire_event({"data": "{0}".format(i)}, "testevents")
                evt = me.get_event(tag="testevents")
                self.assertGotEvent(
                    evt, {"data": "{0}".format(i)}, "Event {0}".format(i)
                )

    @slowTest
    def test_event_many_backlog(self):
        """Test a large number of events, send all then recv all"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            # Must not exceed zmq HWM
            for i in range(500):
                me.fire_event({"data": "{0}".format(i)}, "testevents")
            for i in range(500):
                evt = me.get_event(tag="testevents")
                self.assertGotEvent(
                    evt, {"data": "{0}".format(i)}, "Event {0}".format(i)
                )

    # Test the fire_master function. As it wraps the underlying fire_event,
    # we don't need to perform extensive testing.
    @slowTest
    def test_send_master_event(self):
        """Tests that sending an event through fire_master generates expected event"""
        with eventpublisher_process(self.sock_dir):
            me = salt.utils.event.MasterEvent(self.sock_dir, listen=True)
            data = {"data": "foo1"}
            me.fire_master(data, "test_master")

            evt = me.get_event(tag="fire_master")
            self.assertGotEvent(
                evt,
                {"data": data, "tag": "test_master", "events": None, "pretag": None},
            )


class TestAsyncEventPublisher(AsyncTestCase):
    def get_new_ioloop(self):
        return salt.ext.tornado.ioloop.IOLoop()

    def setUp(self):
        super(TestAsyncEventPublisher, self).setUp()
        self.sock_dir = os.path.join(RUNTIME_VARS.TMP, "test-socks")
        if not os.path.exists(self.sock_dir):
            os.makedirs(self.sock_dir)
        self.addCleanup(shutil.rmtree, self.sock_dir, ignore_errors=True)
        self.opts = {"sock_dir": self.sock_dir}
        self.publisher = salt.utils.event.AsyncEventPublisher(self.opts, self.io_loop,)
        self.event = salt.utils.event.get_event(
            "minion", opts=self.opts, io_loop=self.io_loop
        )
        self.event.subscribe("")
        self.event.set_event_handler(self._handle_publish)

    def _handle_publish(self, raw):
        self.tag, self.data = salt.utils.event.SaltEvent.unpack(raw)
        self.stop()

    def test_event_subscription(self):
        """Test a single event is received"""
        me = salt.utils.event.MinionEvent(self.opts, listen=True)
        me.fire_event({"data": "foo1"}, "evt1")
        self.wait()
        evt1 = me.get_event(tag="evt1")
        self.assertEqual(self.tag, "evt1")
        self.data.pop("_stamp")  # drop the stamp
        self.assertEqual(self.data, {"data": "foo1"})

    def test_event_unsubscribe_remove_error(self):
        me = salt.utils.event.MinionEvent(self.opts, listen=True)
        tag = "evt1"
        me.fire_event({"data": "foo1"}, tag)

        # Make sure no remove error is raised when tag is not found
        for _ in range(2):
            me.unsubscribe(tag)

        me.unsubscribe("tag_does_not_exist")


class TestEventReturn(TestCase):
    @slowTest
    def test_event_return(self):
        evt = None
        try:
            evt = salt.utils.event.EventReturn(salt.config.DEFAULT_MASTER_OPTS.copy())
            evt.start()
        except TypeError as exc:
            if "object" in str(exc):
                self.fail("'{}' TypeError should have not been raised".format(exc))
        finally:
            if evt is not None:
                terminate_process(evt.pid, kill_children=True)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

import logging
import os
import random
import string
from tempfile import NamedTemporaryFile

import salt.config
import salt.loader
import salt.modules.boto_lambda as boto_lambda
import salt.utils.stringutils
from salt.exceptions import SaltInvocationError
from salt.ext import six
from salt.ext.six.moves import range
from salt.utils.versions import LooseVersion
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

# pylint: disable=import-error,no-name-in-module
try:
    import boto3
    from botocore.exceptions import ClientError
    from botocore import __version__ as found_botocore_version

    HAS_BOTO = True
except ImportError:
    HAS_BOTO = False

# pylint: enable=import-error,no-name-in-module

# the boto_lambda module relies on the connect_to_region() method
# which was added in boto 2.8.0
# https://github.com/boto/boto/commit/33ac26b416fbb48a60602542b4ce15dcc7029f12
required_boto3_version = "1.2.1"
required_botocore_version = "1.5.2"

region = "us-east-1"
access_key = "GKTADJGHEIQSXMKKRBJ08H"
secret_key = "askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs"
conn_parameters = {
    "region": region,
    "key": access_key,
    "keyid": secret_key,
    "profile": {},
}
error_message = (
    "An error occurred (101) when calling the {0} operation: Test-defined error"
)
error_content = {"Error": {"Code": 101, "Message": "Test-defined error"}}
function_ret = dict(
    FunctionName="testfunction",
    Runtime="python2.7",
    Role=None,
    Handler="handler",
    Description="abcdefg",
    Timeout=5,
    MemorySize=128,
    CodeSha256="abcdef",
    CodeSize=199,
    FunctionArn="arn:lambda:us-east-1:1234:Something",
    LastModified="yes",
    VpcConfig=None,
    Environment=None,
)
alias_ret = dict(
    AliasArn="arn:lambda:us-east-1:1234:Something",
    Name="testalias",
    FunctionVersion="3",
    Description="Alias description",
)
event_source_mapping_ret = dict(
    UUID="1234-1-123",
    BatchSize=123,
    EventSourceArn="arn:lambda:us-east-1:1234:Something",
    FunctionArn="arn:lambda:us-east-1:1234:Something",
    LastModified="yes",
    LastProcessingResult="SUCCESS",
    State="Enabled",
    StateTransitionReason="Random",
)

log = logging.getLogger(__name__)


def _has_required_boto():
    """
    Returns True/False boolean depending on if Boto is installed and correct
    version.
    """
    if not HAS_BOTO:
        return False
    elif LooseVersion(boto3.__version__) < LooseVersion(required_boto3_version):
        return False
    elif LooseVersion(found_botocore_version) < LooseVersion(required_botocore_version):
        return False
    else:
        return True


@skipIf(HAS_BOTO is False, "The boto module must be installed.")
@skipIf(
    _has_required_boto() is False,
    (
        "The boto3 module must be greater than or equal to version {0}, "
        "and botocore must be greater than or equal to {1}".format(
            required_boto3_version, required_botocore_version
        )
    ),
)
class BotoLambdaTestCaseBase(TestCase, LoaderModuleMockMixin):
    conn = None

    def setup_loader_modules(self):
        self.opts = opts = salt.config.DEFAULT_MINION_OPTS.copy()
        utils = salt.loader.utils(
            opts, whitelist=["boto3", "args", "systemd", "path", "platform"], context={}
        )
        return {boto_lambda: {"__utils__": utils}}

    def setUp(self):
        super(BotoLambdaTestCaseBase, self).setUp()
        boto_lambda.__init__(self.opts)
        del self.opts
        # Set up MagicMock to replace the boto3 session
        # connections keep getting cached from prior tests, can't find the
        # correct context object to clear it. So randomize the cache key, to prevent any
        # cache hits
        conn_parameters["key"] = "".join(
            random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
        )

        self.patcher = patch("boto3.session.Session")
        self.addCleanup(self.patcher.stop)
        self.addCleanup(delattr, self, "patcher")
        mock_session = self.patcher.start()

        session_instance = mock_session.return_value
        self.conn = MagicMock()
        session_instance.client.return_value = self.conn
        self.addCleanup(delattr, self, "conn")


class TempZipFile(object):
    def __enter__(self):
        with NamedTemporaryFile(
            suffix=".zip", prefix="salt_test_", delete=False
        ) as tmp:
            to_write = "###\n"
            if six.PY3:
                to_write = salt.utils.stringutils.to_bytes(to_write)
            tmp.write(to_write)
            self.zipfile = tmp.name
        return self.zipfile

    def __exit__(self, type, value, traceback):
        os.remove(self.zipfile)


class BotoLambdaTestCaseMixin(object):
    pass


class BotoLambdaFunctionTestCase(BotoLambdaTestCaseBase, BotoLambdaTestCaseMixin):
    """
    TestCase for salt.modules.boto_lambda module
    """

    def test_that_when_checking_if_a_function_exists_and_a_function_exists_the_function_exists_method_returns_true(
        self,
    ):
        """
        Tests checking lambda function existence when the lambda function already exists
        """
        self.conn.list_functions.return_value = {"Functions": [function_ret]}
        func_exists_result = boto_lambda.function_exists(
            FunctionName=function_ret["FunctionName"], **conn_parameters
        )

        self.assertTrue(func_exists_result["exists"])

    def test_that_when_checking_if_a_function_exists_and_a_function_does_not_exist_the_function_exists_method_returns_false(
        self,
    ):
        """
        Tests checking lambda function existence when the lambda function does not exist
        """
        self.conn.list_functions.return_value = {"Functions": [function_ret]}
        func_exists_result = boto_lambda.function_exists(
            FunctionName="myfunc", **conn_parameters
        )

        self.assertFalse(func_exists_result["exists"])

    def test_that_when_checking_if_a_function_exists_and_boto3_returns_an_error_the_function_exists_method_returns_error(
        self,
    ):
        """
        Tests checking lambda function existence when boto returns an error
        """
        self.conn.list_functions.side_effect = ClientError(
            error_content, "list_functions"
        )
        func_exists_result = boto_lambda.function_exists(
            FunctionName="myfunc", **conn_parameters
        )

        self.assertEqual(
            func_exists_result.get("error", {}).get("message"),
            error_message.format("list_functions"),
        )

    def test_that_when_creating_a_function_from_zipfile_succeeds_the_create_function_method_returns_true(
        self,
    ):
        """
        tests True function created.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            with TempZipFile() as zipfile:
                self.conn.create_function.return_value = function_ret
                lambda_creation_result = boto_lambda.create_function(
                    FunctionName="testfunction",
                    Runtime="python2.7",
                    Role="myrole",
                    Handler="file.method",
                    ZipFile=zipfile,
                    **conn_parameters
                )

        self.assertTrue(lambda_creation_result["created"])

    def test_that_when_creating_a_function_from_s3_succeeds_the_create_function_method_returns_true(
        self,
    ):
        """
        tests True function created.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            self.conn.create_function.return_value = function_ret
            lambda_creation_result = boto_lambda.create_function(
                FunctionName="testfunction",
                Runtime="python2.7",
                Role="myrole",
                Handler="file.method",
                S3Bucket="bucket",
                S3Key="key",
                **conn_parameters
            )

        self.assertTrue(lambda_creation_result["created"])

    def test_that_when_creating_a_function_without_code_raises_a_salt_invocation_error(
        self,
    ):
        """
        tests Creating a function without code
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            with self.assertRaisesRegex(
                SaltInvocationError,
                "Either ZipFile must be specified, or S3Bucket and S3Key must be provided.",
            ):
                lambda_creation_result = boto_lambda.create_function(
                    FunctionName="testfunction",
                    Runtime="python2.7",
                    Role="myrole",
                    Handler="file.method",
                    **conn_parameters
                )

    def test_that_when_creating_a_function_with_zipfile_and_s3_raises_a_salt_invocation_error(
        self,
    ):
        """
        tests Creating a function without code
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            with self.assertRaisesRegex(
                SaltInvocationError,
                "Either ZipFile must be specified, or S3Bucket and S3Key must be provided.",
            ):
                with TempZipFile() as zipfile:
                    lambda_creation_result = boto_lambda.create_function(
                        FunctionName="testfunction",
                        Runtime="python2.7",
                        Role="myrole",
                        Handler="file.method",
                        ZipFile=zipfile,
                        S3Bucket="bucket",
                        S3Key="key",
                        **conn_parameters
                    )

    def test_that_when_creating_a_function_fails_the_create_function_method_returns_error(
        self,
    ):
        """
        tests False function not created.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            self.conn.create_function.side_effect = ClientError(
                error_content, "create_function"
            )
            with TempZipFile() as zipfile:
                lambda_creation_result = boto_lambda.create_function(
                    FunctionName="testfunction",
                    Runtime="python2.7",
                    Role="myrole",
                    Handler="file.method",
                    ZipFile=zipfile,
                    **conn_parameters
                )
        self.assertEqual(
            lambda_creation_result.get("error", {}).get("message"),
            error_message.format("create_function"),
        )

    def test_that_when_deleting_a_function_succeeds_the_delete_function_method_returns_true(
        self,
    ):
        """
        tests True function deleted.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            result = boto_lambda.delete_function(
                FunctionName="testfunction", Qualifier=1, **conn_parameters
            )

        self.assertTrue(result["deleted"])

    def test_that_when_deleting_a_function_fails_the_delete_function_method_returns_false(
        self,
    ):
        """
        tests False function not deleted.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            self.conn.delete_function.side_effect = ClientError(
                error_content, "delete_function"
            )
            result = boto_lambda.delete_function(
                FunctionName="testfunction", **conn_parameters
            )
        self.assertFalse(result["deleted"])

    def test_that_when_describing_function_it_returns_the_dict_of_properties_returns_true(
        self,
    ):
        """
        Tests describing parameters if function exists
        """
        self.conn.list_functions.return_value = {"Functions": [function_ret]}

        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            result = boto_lambda.describe_function(
                FunctionName=function_ret["FunctionName"], **conn_parameters
            )

        self.assertEqual(result, {"function": function_ret})

    def test_that_when_describing_function_it_returns_the_dict_of_properties_returns_false(
        self,
    ):
        """
        Tests describing parameters if function does not exist
        """
        self.conn.list_functions.return_value = {"Functions": []}
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            result = boto_lambda.describe_function(
                FunctionName="testfunction", **conn_parameters
            )

        self.assertFalse(result["function"])

    def test_that_when_describing_lambda_on_client_error_it_returns_error(self):
        """
        Tests describing parameters failure
        """
        self.conn.list_functions.side_effect = ClientError(
            error_content, "list_functions"
        )
        result = boto_lambda.describe_function(
            FunctionName="testfunction", **conn_parameters
        )
        self.assertTrue("error" in result)

    def test_that_when_updating_a_function_succeeds_the_update_function_method_returns_true(
        self,
    ):
        """
        tests True function updated.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            self.conn.update_function_config.return_value = function_ret
            result = boto_lambda.update_function_config(
                FunctionName=function_ret["FunctionName"],
                Role="myrole",
                **conn_parameters
            )

        self.assertTrue(result["updated"])

    def test_that_when_updating_a_function_fails_the_update_function_method_returns_error(
        self,
    ):
        """
        tests False function not updated.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            self.conn.update_function_configuration.side_effect = ClientError(
                error_content, "update_function"
            )
            result = boto_lambda.update_function_config(
                FunctionName="testfunction", Role="myrole", **conn_parameters
            )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("update_function"),
        )

    def test_that_when_updating_function_code_from_zipfile_succeeds_the_update_function_method_returns_true(
        self,
    ):
        """
        tests True function updated.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            with TempZipFile() as zipfile:
                self.conn.update_function_code.return_value = function_ret
                result = boto_lambda.update_function_code(
                    FunctionName=function_ret["FunctionName"],
                    ZipFile=zipfile,
                    **conn_parameters
                )

        self.assertTrue(result["updated"])

    def test_that_when_updating_function_code_from_s3_succeeds_the_update_function_method_returns_true(
        self,
    ):
        """
        tests True function updated.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            self.conn.update_function_code.return_value = function_ret
            result = boto_lambda.update_function_code(
                FunctionName="testfunction",
                S3Bucket="bucket",
                S3Key="key",
                **conn_parameters
            )

        self.assertTrue(result["updated"])

    def test_that_when_updating_function_code_without_code_raises_a_salt_invocation_error(
        self,
    ):
        """
        tests Creating a function without code
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            with self.assertRaisesRegex(
                SaltInvocationError,
                (
                    "Either ZipFile must be specified, or S3Bucket "
                    "and S3Key must be provided."
                ),
            ):
                result = boto_lambda.update_function_code(
                    FunctionName="testfunction", **conn_parameters
                )

    def test_that_when_updating_function_code_fails_the_update_function_method_returns_error(
        self,
    ):
        """
        tests False function not updated.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            self.conn.update_function_code.side_effect = ClientError(
                error_content, "update_function_code"
            )
            result = boto_lambda.update_function_code(
                FunctionName="testfunction",
                S3Bucket="bucket",
                S3Key="key",
                **conn_parameters
            )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("update_function_code"),
        )

    def test_that_when_listing_function_versions_succeeds_the_list_function_versions_method_returns_true(
        self,
    ):
        """
        tests True function versions listed.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            self.conn.list_versions_by_function.return_value = {
                "Versions": [function_ret]
            }
            result = boto_lambda.list_function_versions(
                FunctionName="testfunction", **conn_parameters
            )

        self.assertTrue(result["Versions"])

    def test_that_when_listing_function_versions_fails_the_list_function_versions_method_returns_false(
        self,
    ):
        """
        tests False no function versions listed.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            self.conn.list_versions_by_function.return_value = {"Versions": []}
            result = boto_lambda.list_function_versions(
                FunctionName="testfunction", **conn_parameters
            )
        self.assertFalse(result["Versions"])

    def test_that_when_listing_function_versions_fails_the_list_function_versions_method_returns_error(
        self,
    ):
        """
        tests False function versions error.
        """
        with patch.dict(
            boto_lambda.__salt__,
            {"boto_iam.get_account_id": MagicMock(return_value="1234")},
        ):
            self.conn.list_versions_by_function.side_effect = ClientError(
                error_content, "list_versions_by_function"
            )
            result = boto_lambda.list_function_versions(
                FunctionName="testfunction", **conn_parameters
            )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("list_versions_by_function"),
        )


@skipIf(HAS_BOTO is False, "The boto module must be installed.")
@skipIf(
    _has_required_boto() is False,
    "The boto3 module must be greater than"
    " or equal to version {0}".format(required_boto3_version),
)
class BotoLambdaAliasTestCase(BotoLambdaTestCaseBase, BotoLambdaTestCaseMixin):
    """
    TestCase for salt.modules.boto_lambda module aliases
    """

    def test_that_when_creating_an_alias_succeeds_the_create_alias_method_returns_true(
        self,
    ):
        """
        tests True alias created.
        """
        self.conn.create_alias.return_value = alias_ret
        result = boto_lambda.create_alias(
            FunctionName="testfunction",
            Name=alias_ret["Name"],
            FunctionVersion=alias_ret["FunctionVersion"],
            **conn_parameters
        )

        self.assertTrue(result["created"])

    def test_that_when_creating_an_alias_fails_the_create_alias_method_returns_error(
        self,
    ):
        """
        tests False alias not created.
        """
        self.conn.create_alias.side_effect = ClientError(error_content, "create_alias")
        result = boto_lambda.create_alias(
            FunctionName="testfunction",
            Name=alias_ret["Name"],
            FunctionVersion=alias_ret["FunctionVersion"],
            **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"), error_message.format("create_alias")
        )

    def test_that_when_deleting_an_alias_succeeds_the_delete_alias_method_returns_true(
        self,
    ):
        """
        tests True alias deleted.
        """
        result = boto_lambda.delete_alias(
            FunctionName="testfunction", Name=alias_ret["Name"], **conn_parameters
        )

        self.assertTrue(result["deleted"])

    def test_that_when_deleting_an_alias_fails_the_delete_alias_method_returns_false(
        self,
    ):
        """
        tests False alias not deleted.
        """
        self.conn.delete_alias.side_effect = ClientError(error_content, "delete_alias")
        result = boto_lambda.delete_alias(
            FunctionName="testfunction", Name=alias_ret["Name"], **conn_parameters
        )
        self.assertFalse(result["deleted"])

    def test_that_when_checking_if_an_alias_exists_and_the_alias_exists_the_alias_exists_method_returns_true(
        self,
    ):
        """
        Tests checking lambda alias existence when the lambda alias already exists
        """
        self.conn.list_aliases.return_value = {"Aliases": [alias_ret]}
        result = boto_lambda.alias_exists(
            FunctionName="testfunction", Name=alias_ret["Name"], **conn_parameters
        )
        self.assertTrue(result["exists"])

    def test_that_when_checking_if_an_alias_exists_and_the_alias_does_not_exist_the_alias_exists_method_returns_false(
        self,
    ):
        """
        Tests checking lambda alias existence when the lambda alias does not exist
        """
        self.conn.list_aliases.return_value = {"Aliases": [alias_ret]}
        result = boto_lambda.alias_exists(
            FunctionName="testfunction", Name="otheralias", **conn_parameters
        )

        self.assertFalse(result["exists"])

    def test_that_when_checking_if_an_alias_exists_and_boto3_returns_an_error_the_alias_exists_method_returns_error(
        self,
    ):
        """
        Tests checking lambda alias existence when boto returns an error
        """
        self.conn.list_aliases.side_effect = ClientError(error_content, "list_aliases")
        result = boto_lambda.alias_exists(
            FunctionName="testfunction", Name=alias_ret["Name"], **conn_parameters
        )

        self.assertEqual(
            result.get("error", {}).get("message"), error_message.format("list_aliases")
        )

    def test_that_when_describing_alias_it_returns_the_dict_of_properties_returns_true(
        self,
    ):
        """
        Tests describing parameters if alias exists
        """
        self.conn.list_aliases.return_value = {"Aliases": [alias_ret]}

        result = boto_lambda.describe_alias(
            FunctionName="testfunction", Name=alias_ret["Name"], **conn_parameters
        )

        self.assertEqual(result, {"alias": alias_ret})

    def test_that_when_describing_alias_it_returns_the_dict_of_properties_returns_false(
        self,
    ):
        """
        Tests describing parameters if alias does not exist
        """
        self.conn.list_aliases.return_value = {"Aliases": [alias_ret]}
        result = boto_lambda.describe_alias(
            FunctionName="testfunction", Name="othername", **conn_parameters
        )

        self.assertFalse(result["alias"])

    def test_that_when_describing_lambda_on_client_error_it_returns_error(self):
        """
        Tests describing parameters failure
        """
        self.conn.list_aliases.side_effect = ClientError(error_content, "list_aliases")
        result = boto_lambda.describe_alias(
            FunctionName="testfunction", Name=alias_ret["Name"], **conn_parameters
        )
        self.assertTrue("error" in result)

    def test_that_when_updating_an_alias_succeeds_the_update_alias_method_returns_true(
        self,
    ):
        """
        tests True alias updated.
        """
        self.conn.update_alias.return_value = alias_ret
        result = boto_lambda.update_alias(
            FunctionName="testfunctoin",
            Name=alias_ret["Name"],
            Description=alias_ret["Description"],
            **conn_parameters
        )

        self.assertTrue(result["updated"])

    def test_that_when_updating_an_alias_fails_the_update_alias_method_returns_error(
        self,
    ):
        """
        tests False alias not updated.
        """
        self.conn.update_alias.side_effect = ClientError(error_content, "update_alias")
        result = boto_lambda.update_alias(
            FunctionName="testfunction", Name=alias_ret["Name"], **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"), error_message.format("update_alias")
        )


@skipIf(HAS_BOTO is False, "The boto module must be installed.")
@skipIf(
    _has_required_boto() is False,
    "The boto3 module must be greater than"
    " or equal to version {0}".format(required_boto3_version),
)
class BotoLambdaEventSourceMappingTestCase(
    BotoLambdaTestCaseBase, BotoLambdaTestCaseMixin
):
    """
    TestCase for salt.modules.boto_lambda module mappings
    """

    def test_that_when_creating_a_mapping_succeeds_the_create_event_source_mapping_method_returns_true(
        self,
    ):
        """
        tests True mapping created.
        """
        self.conn.create_event_source_mapping.return_value = event_source_mapping_ret
        result = boto_lambda.create_event_source_mapping(
            EventSourceArn=event_source_mapping_ret["EventSourceArn"],
            FunctionName=event_source_mapping_ret["FunctionArn"],
            StartingPosition="LATEST",
            **conn_parameters
        )

        self.assertTrue(result["created"])

    def test_that_when_creating_an_event_source_mapping_fails_the_create_event_source_mapping_method_returns_error(
        self,
    ):
        """
        tests False mapping not created.
        """
        self.conn.create_event_source_mapping.side_effect = ClientError(
            error_content, "create_event_source_mapping"
        )
        result = boto_lambda.create_event_source_mapping(
            EventSourceArn=event_source_mapping_ret["EventSourceArn"],
            FunctionName=event_source_mapping_ret["FunctionArn"],
            StartingPosition="LATEST",
            **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("create_event_source_mapping"),
        )

    def test_that_when_listing_mapping_ids_succeeds_the_get_event_source_mapping_ids_method_returns_true(
        self,
    ):
        """
        tests True mapping ids listed.
        """
        self.conn.list_event_source_mappings.return_value = {
            "EventSourceMappings": [event_source_mapping_ret]
        }
        result = boto_lambda.get_event_source_mapping_ids(
            EventSourceArn=event_source_mapping_ret["EventSourceArn"],
            FunctionName=event_source_mapping_ret["FunctionArn"],
            **conn_parameters
        )

        self.assertTrue(result)

    def test_that_when_listing_event_source_mapping_ids_fails_the_get_event_source_mapping_ids_versions_method_returns_false(
        self,
    ):
        """
        tests False no mapping ids listed.
        """
        self.conn.list_event_source_mappings.return_value = {"EventSourceMappings": []}
        result = boto_lambda.get_event_source_mapping_ids(
            EventSourceArn=event_source_mapping_ret["EventSourceArn"],
            FunctionName=event_source_mapping_ret["FunctionArn"],
            **conn_parameters
        )
        self.assertFalse(result)

    def test_that_when_listing_event_source_mapping_ids_fails_the_get_event_source_mapping_ids_method_returns_error(
        self,
    ):
        """
        tests False mapping ids error.
        """
        self.conn.list_event_source_mappings.side_effect = ClientError(
            error_content, "list_event_source_mappings"
        )
        result = boto_lambda.get_event_source_mapping_ids(
            EventSourceArn=event_source_mapping_ret["EventSourceArn"],
            FunctionName=event_source_mapping_ret["FunctionArn"],
            **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("list_event_source_mappings"),
        )

    def test_that_when_deleting_an_event_source_mapping_by_UUID_succeeds_the_delete_event_source_mapping_method_returns_true(
        self,
    ):
        """
        tests True mapping deleted.
        """
        result = boto_lambda.delete_event_source_mapping(
            UUID=event_source_mapping_ret["UUID"], **conn_parameters
        )
        self.assertTrue(result["deleted"])

    @skipIf(True, "This appears to leak memory and crash the unit test suite")
    def test_that_when_deleting_an_event_source_mapping_by_name_succeeds_the_delete_event_source_mapping_method_returns_true(
        self,
    ):
        """
        tests True mapping deleted.
        """
        self.conn.list_event_source_mappings.return_value = {
            "EventSourceMappings": [event_source_mapping_ret]
        }
        result = boto_lambda.delete_event_source_mapping(
            EventSourceArn=event_source_mapping_ret["EventSourceArn"],
            FunctionName=event_source_mapping_ret["FunctionArn"],
            **conn_parameters
        )
        self.assertTrue(result["deleted"])

    def test_that_when_deleting_an_event_source_mapping_without_identifier_the_delete_event_source_mapping_method_raises_saltinvocationexception(
        self,
    ):
        """
        tests Deleting a mapping without identifier
        """
        with self.assertRaisesRegex(
            SaltInvocationError,
            (
                "Either UUID must be specified, or EventSourceArn "
                "and FunctionName must be provided."
            ),
        ):
            result = boto_lambda.delete_event_source_mapping(**conn_parameters)

    def test_that_when_deleting_an_event_source_mapping_fails_the_delete_event_source_mapping_method_returns_false(
        self,
    ):
        """
        tests False mapping not deleted.
        """
        self.conn.delete_event_source_mapping.side_effect = ClientError(
            error_content, "delete_event_source_mapping"
        )
        result = boto_lambda.delete_event_source_mapping(
            UUID=event_source_mapping_ret["UUID"], **conn_parameters
        )
        self.assertFalse(result["deleted"])

    def test_that_when_checking_if_an_event_source_mapping_exists_and_the_event_source_mapping_exists_the_event_source_mapping_exists_method_returns_true(
        self,
    ):
        """
        Tests checking lambda event_source_mapping existence when the lambda
        event_source_mapping already exists
        """
        self.conn.get_event_source_mapping.return_value = event_source_mapping_ret
        result = boto_lambda.event_source_mapping_exists(
            UUID=event_source_mapping_ret["UUID"], **conn_parameters
        )
        self.assertTrue(result["exists"])

    def test_that_when_checking_if_an_event_source_mapping_exists_and_the_event_source_mapping_does_not_exist_the_event_source_mapping_exists_method_returns_false(
        self,
    ):
        """
        Tests checking lambda event_source_mapping existence when the lambda
        event_source_mapping does not exist
        """
        self.conn.get_event_source_mapping.return_value = None
        result = boto_lambda.event_source_mapping_exists(
            UUID="other_UUID", **conn_parameters
        )
        self.assertFalse(result["exists"])

    def test_that_when_checking_if_an_event_source_mapping_exists_and_boto3_returns_an_error_the_event_source_mapping_exists_method_returns_error(
        self,
    ):
        """
        Tests checking lambda event_source_mapping existence when boto returns an error
        """
        self.conn.get_event_source_mapping.side_effect = ClientError(
            error_content, "list_event_source_mappings"
        )
        result = boto_lambda.event_source_mapping_exists(
            UUID=event_source_mapping_ret["UUID"], **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("list_event_source_mappings"),
        )

    def test_that_when_describing_event_source_mapping_it_returns_the_dict_of_properties_returns_true(
        self,
    ):
        """
        Tests describing parameters if event_source_mapping exists
        """
        self.conn.get_event_source_mapping.return_value = event_source_mapping_ret
        result = boto_lambda.describe_event_source_mapping(
            UUID=event_source_mapping_ret["UUID"], **conn_parameters
        )
        self.assertEqual(result, {"event_source_mapping": event_source_mapping_ret})

    def test_that_when_describing_event_source_mapping_it_returns_the_dict_of_properties_returns_false(
        self,
    ):
        """
        Tests describing parameters if event_source_mapping does not exist
        """
        self.conn.get_event_source_mapping.return_value = None
        result = boto_lambda.describe_event_source_mapping(
            UUID=event_source_mapping_ret["UUID"], **conn_parameters
        )
        self.assertFalse(result["event_source_mapping"])

    def test_that_when_describing_event_source_mapping_on_client_error_it_returns_error(
        self,
    ):
        """
        Tests describing parameters failure
        """
        self.conn.get_event_source_mapping.side_effect = ClientError(
            error_content, "get_event_source_mapping"
        )
        result = boto_lambda.describe_event_source_mapping(
            UUID=event_source_mapping_ret["UUID"], **conn_parameters
        )
        self.assertTrue("error" in result)

    def test_that_when_updating_an_event_source_mapping_succeeds_the_update_event_source_mapping_method_returns_true(
        self,
    ):
        """
        tests True event_source_mapping updated.
        """
        self.conn.update_event_source_mapping.return_value = event_source_mapping_ret
        result = boto_lambda.update_event_source_mapping(
            UUID=event_source_mapping_ret["UUID"],
            FunctionName=event_source_mapping_ret["FunctionArn"],
            **conn_parameters
        )

        self.assertTrue(result["updated"])

    def test_that_when_updating_an_event_source_mapping_fails_the_update_event_source_mapping_method_returns_error(
        self,
    ):
        """
        tests False event_source_mapping not updated.
        """
        self.conn.update_event_source_mapping.side_effect = ClientError(
            error_content, "update_event_source_mapping"
        )
        result = boto_lambda.update_event_source_mapping(
            UUID=event_source_mapping_ret["UUID"],
            FunctionName=event_source_mapping_ret["FunctionArn"],
            **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("update_event_source_mapping"),
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.aws_sqs as aws_sqs

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class AwsSqsTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.aws_sqs
    """

    def setup_loader_modules(self):
        return {aws_sqs: {}}

    # 'exists' function tests: 1

    def test_exists(self):
        """
        Test to ensure the SQS queue exists.
        """
        name = "myqueue"
        region = "eu-west-1"

        ret = {"name": name, "result": None, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[False, True])
        with patch.dict(aws_sqs.__salt__, {"aws_sqs.queue_exists": mock}):
            comt = "AWS SQS queue {0} is set to be created".format(name)
            ret.update({"comment": comt})
            with patch.dict(aws_sqs.__opts__, {"test": True}):
                self.assertDictEqual(aws_sqs.exists(name, region), ret)

            comt = "{0} exists in {1}".format(name, region)
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(aws_sqs.exists(name, region), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to remove the named SQS queue if it exists.
        """
        name = "myqueue"
        region = "eu-west-1"

        ret = {"name": name, "result": None, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[True, False])
        with patch.dict(aws_sqs.__salt__, {"aws_sqs.queue_exists": mock}):
            comt = "AWS SQS queue {0} is set to be removed".format(name)
            ret.update({"comment": comt})
            with patch.dict(aws_sqs.__opts__, {"test": True}):
                self.assertDictEqual(aws_sqs.absent(name, region), ret)

            comt = "{0} does not exist in {1}".format(name, region)
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(aws_sqs.absent(name, region), ret)



# -*- coding: utf-8 -*-
"""
:codeauthor: Shane Lee <slee@saltstack.com>
"""
# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.config

# Import 3rd Party Libs
import salt.ext.six as six
import salt.loader
import salt.states.win_lgpo as win_lgpo
import salt.utils.platform
import salt.utils.stringutils

# Import Salt Testing Libs
from tests.support.helpers import destructiveTest, slowTest
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import patch
from tests.support.unit import TestCase, skipIf

# We're going to actually use the loader, without grains (slow)
opts = salt.config.DEFAULT_MINION_OPTS.copy()
utils = salt.loader.utils(opts)
modules = salt.loader.minion_mods(opts, utils=utils)

LOADER_DICTS = {win_lgpo: {"__opts__": opts, "__salt__": modules, "__utils__": utils}}


class WinLGPOComparePoliciesTestCase(TestCase):
    """
    Test cases for the win_lgpo state
    """

    def test__compare_policies_string(self):
        """
        ``_compare_policies`` should only return ``True`` when the string values
        are the same. All other scenarios should return ``False``
        """
        compare_string = "Salty test"
        # Same
        self.assertTrue(win_lgpo._compare_policies(compare_string, compare_string))
        # Different
        self.assertFalse(win_lgpo._compare_policies(compare_string, "Not the same"))
        # List
        self.assertFalse(win_lgpo._compare_policies(compare_string, ["item1", "item2"]))
        # Dict
        self.assertFalse(win_lgpo._compare_policies(compare_string, {"key": "value"}))
        # None
        self.assertFalse(win_lgpo._compare_policies(compare_string, None))

    def test__compare_policies_list(self):
        """
        ``_compare_policies`` should only return ``True`` when the lists are the
        same. All other scenarios should return ``False``
        """
        compare_list = ["Salty", "test"]
        # Same
        self.assertTrue(win_lgpo._compare_policies(compare_list, compare_list))
        # Different
        self.assertFalse(
            win_lgpo._compare_policies(compare_list, ["Not", "the", "same"])
        )
        # String
        self.assertFalse(win_lgpo._compare_policies(compare_list, "Not a list"))
        # Dict
        self.assertFalse(win_lgpo._compare_policies(compare_list, {"key": "value"}))
        # None
        self.assertFalse(win_lgpo._compare_policies(compare_list, None))

    def test__compare_policies_dict(self):
        """
        ``_compare_policies`` should only return ``True`` when the dicts are the
        same. All other scenarios should return ``False``
        """
        compare_dict = {"Salty": "test"}
        # Same
        self.assertTrue(win_lgpo._compare_policies(compare_dict, compare_dict))
        # Different
        self.assertFalse(win_lgpo._compare_policies(compare_dict, {"key": "value"}))
        # String
        self.assertFalse(win_lgpo._compare_policies(compare_dict, "Not a dict"))
        # List
        self.assertFalse(win_lgpo._compare_policies(compare_dict, ["Not", "a", "dict"]))
        # None
        self.assertFalse(win_lgpo._compare_policies(compare_dict, None))

    def test__compare_policies_integer(self):
        """
        ``_compare_policies`` should only return ``True`` when the integer
        values are the same. All other scenarios should return ``False``
        """
        compare_integer = 1
        # Same
        self.assertTrue(win_lgpo._compare_policies(compare_integer, compare_integer))
        # Different
        self.assertFalse(win_lgpo._compare_policies(compare_integer, 0))
        # List
        self.assertFalse(
            win_lgpo._compare_policies(compare_integer, ["item1", "item2"])
        )
        # Dict
        self.assertFalse(win_lgpo._compare_policies(compare_integer, {"key": "value"}))
        # None
        self.assertFalse(win_lgpo._compare_policies(compare_integer, None))


@destructiveTest
@skipIf(not salt.utils.platform.is_windows(), "System is not Windows")
class WinLGPOPolicyElementNames(TestCase, LoaderModuleMockMixin):
    """
    Test variations of the Point and Print Restrictions policy when Not
    Configured (NC)
    """

    def setup_loader_modules(self):
        return LOADER_DICTS

    def setUp(self):
        computer_policy = {"Point and Print Restrictions": "Not Configured"}
        with patch.dict(win_lgpo.__opts__, {"test": False}):
            win_lgpo.set_(name="nc_state", computer_policy=computer_policy)

    @slowTest
    def test_current_element_naming_style(self):
        computer_policy = {
            "Point and Print Restrictions": {
                "Users can only point and print to these servers": True,
                "Enter fully qualified server names separated by "
                "semicolons": "fakeserver1;fakeserver2",
                "Users can only point and print to machines in their " "forest": True,
                "When installing drivers for a new connection": "Show warning and elevation prompt",
                "When updating drivers for an existing connection": "Show warning only",
            }
        }
        with patch.dict(win_lgpo.__opts__, {"test": False}):
            result = win_lgpo.set_(name="test_state", computer_policy=computer_policy)
            result = win_lgpo._convert_to_unicode(result)
        expected = {
            "Point and Print Restrictions": {
                "Enter fully qualified server names separated by "
                "semicolons": "fakeserver1;fakeserver2",
                "When installing drivers for a new connection": "Show warning and elevation prompt",
                "Users can only point and print to machines in " "their forest": True,
                "Users can only point and print to these servers": True,
                "When updating drivers for an existing connection": "Show warning only",
            }
        }
        self.assertDictEqual(
            result["changes"]["new"]["Computer Configuration"], expected
        )

    @slowTest
    def test_old_element_naming_style(self):
        computer_policy = {
            "Point and Print Restrictions": {
                "Users can only point and print to these servers": True,
                "Enter fully qualified server names separated by "
                "semicolons": "fakeserver1;fakeserver2",
                "Users can only point and print to machines in their " "forest": True,
                # Here's the old one
                "Security Prompts: When installing drivers for a new connection": "Show warning and elevation prompt",
                "When updating drivers for an existing connection": "Show warning only",
            }
        }

        with patch.dict(win_lgpo.__opts__, {"test": False}):
            result = win_lgpo.set_(name="test_state", computer_policy=computer_policy)
            if six.PY2:
                result = win_lgpo._convert_to_unicode(result)
        expected = {
            "Point and Print Restrictions": {
                "Enter fully qualified server names separated by "
                "semicolons": "fakeserver1;fakeserver2",
                "When installing drivers for a new connection": "Show warning and elevation prompt",
                "Users can only point and print to machines in " "their forest": True,
                "Users can only point and print to these servers": True,
                "When updating drivers for an existing connection": "Show warning only",
            }
        }
        self.assertDictEqual(
            result["changes"]["new"]["Computer Configuration"], expected
        )
        expected = (
            "The LGPO module changed the way it gets policy element names.\n"
            '"Security Prompts: When installing drivers for a new connection" is no longer valid.\n'
            'Please use "When installing drivers for a new connection" instead.\n'
            "The following policies changed:\n"
            "Point and Print Restrictions"
        )
        self.assertEqual(result["comment"], expected)

    @slowTest
    def test_invalid_elements(self):
        computer_policy = {
            "Point and Print Restrictions": {
                "Invalid element spongebob": True,
                "Invalid element squidward": False,
            }
        }

        with patch.dict(win_lgpo.__opts__, {"test": False}):
            result = win_lgpo.set_(name="test_state", computer_policy=computer_policy)
        expected = {
            "changes": {},
            "comment": "Invalid element name: Invalid element squidward\n"
            "Invalid element name: Invalid element spongebob",
            "name": "test_state",
            "result": False,
        }
        self.assertDictEqual(result["changes"], expected["changes"])
        self.assertIn("Invalid element squidward", result["comment"])
        self.assertIn("Invalid element spongebob", result["comment"])
        self.assertFalse(expected["result"])


@destructiveTest
@skipIf(not salt.utils.platform.is_windows(), "System is not Windows")
class WinLGPOPolicyElementNamesTestTrue(TestCase, LoaderModuleMockMixin):
    """
    Test variations of the Point and Print Restrictions policy when Not
    Configured (NC)
    """

    configured = False

    def setup_loader_modules(self):
        return LOADER_DICTS

    def setUp(self):
        if not self.configured:
            computer_policy = {
                "Point and Print Restrictions": {
                    "Users can only point and print to these servers": True,
                    "Enter fully qualified server names separated by "
                    "semicolons": "fakeserver1;fakeserver2",
                    "Users can only point and print to machines in their "
                    "forest": True,
                    "When installing drivers for a new connection": "Show warning and elevation prompt",
                    "When updating drivers for an existing connection": "Show warning only",
                }
            }
            with patch.dict(win_lgpo.__opts__, {"test": False}):
                win_lgpo.set_(name="nc_state", computer_policy=computer_policy)
            self.configured = True

    @slowTest
    def test_current_element_naming_style(self):
        computer_policy = {
            "Point and Print Restrictions": {
                "Users can only point and print to these servers": True,
                "Enter fully qualified server names separated by "
                "semicolons": "fakeserver1;fakeserver2",
                "Users can only point and print to machines in their " "forest": True,
                "When installing drivers for a new connection": "Show warning and elevation prompt",
                "When updating drivers for an existing connection": "Show warning only",
            }
        }
        with patch.dict(win_lgpo.__opts__, {"test": True}):
            result = win_lgpo.set_(name="test_state", computer_policy=computer_policy)
        expected = {
            "changes": {},
            "comment": "All specified policies are properly configured",
        }
        self.assertDictEqual(result["changes"], expected["changes"])
        self.assertTrue(result["result"])
        self.assertEqual(result["comment"], expected["comment"])

    @slowTest
    def test_old_element_naming_style(self):
        computer_policy = {
            "Point and Print Restrictions": {
                "Users can only point and print to these servers": True,
                "Enter fully qualified server names separated by "
                "semicolons": "fakeserver1;fakeserver2",
                "Users can only point and print to machines in their " "forest": True,
                # Here's the old one
                "Security Prompts: When installing drivers for a new connection": "Show warning and elevation prompt",
                "When updating drivers for an existing connection": "Show warning only",
            }
        }
        with patch.dict(win_lgpo.__opts__, {"test": True}):
            result = win_lgpo.set_(name="test_state", computer_policy=computer_policy)
        expected = {
            "changes": {},
            "comment": "The LGPO module changed the way it gets policy element names.\n"
            '"Security Prompts: When installing drivers for a new connection" is no longer valid.\n'
            'Please use "When installing drivers for a new connection" instead.\n'
            "All specified policies are properly configured",
        }
        self.assertDictEqual(result["changes"], expected["changes"])
        self.assertTrue(result["result"])
        self.assertEqual(result["comment"], expected["comment"])

    @slowTest
    def test_invalid_elements(self):
        computer_policy = {
            "Point and Print Restrictions": {
                "Invalid element spongebob": True,
                "Invalid element squidward": False,
            }
        }

        with patch.dict(win_lgpo.__opts__, {"test": True}):
            result = win_lgpo.set_(name="test_state", computer_policy=computer_policy)
        expected = {
            "changes": {},
            "comment": "Invalid element name: Invalid element squidward\n"
            "Invalid element name: Invalid element spongebob",
            "name": "test_state",
            "result": False,
        }
        self.assertDictEqual(result["changes"], expected["changes"])
        self.assertIn("Invalid element squidward", result["comment"])
        self.assertIn("Invalid element spongebob", result["comment"])
        self.assertFalse(expected["result"])



# -*- coding: utf-8 -*-
"""
    :codeauthor: Denys Havrysh <denys.gavrysh@gmail.com>
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import os
import shutil
import tempfile

import salt.config

# Import Salt Libs
import salt.log.setup as log
import salt.syspaths
import salt.utils.parsers
import salt.utils.platform
from tests.support.mock import MagicMock, patch
from tests.support.runtests import RUNTIME_VARS

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf


class ErrorMock(object):  # pylint: disable=too-few-public-methods
    """
    Error handling
    """

    def __init__(self):
        """
        init
        """
        self.msg = None

    def error(self, msg):
        """
        Capture error message
        """
        self.msg = msg


class LogSetupMock(object):
    """
    Logger setup
    """

    def __init__(self):
        """
        init
        """
        self.log_level = None
        self.log_file = None
        self.log_level_logfile = None
        self.config = {}
        self.temp_log_level = None

    def setup_console_logger(
        self, log_level="error", **kwargs
    ):  # pylint: disable=unused-argument
        """
        Set console loglevel
        """
        self.log_level = log_level

    def setup_extended_logging(self, opts):
        """
        Set opts
        """
        self.config = opts

    def setup_logfile_logger(
        self, logfile, loglevel, **kwargs
    ):  # pylint: disable=unused-argument
        """
        Set logfile and loglevel
        """
        self.log_file = logfile
        self.log_level_logfile = loglevel

    @staticmethod
    def get_multiprocessing_logging_queue():  # pylint: disable=invalid-name
        """
        Mock
        """
        import multiprocessing

        return multiprocessing.Queue()

    def setup_multiprocessing_logging_listener(
        self, opts, *args
    ):  # pylint: disable=invalid-name,unused-argument
        """
        Set opts
        """
        self.config = opts

    def setup_temp_logger(self, log_level="error"):
        """
        Set temp loglevel
        """
        self.temp_log_level = log_level


class ObjectView(object):  # pylint: disable=too-few-public-methods
    """
    Dict object view
    """

    def __init__(self, d):
        self.__dict__ = d


class ParserBase(object):
    """
    Unit Tests for Log Level Mixin with Salt parsers
    """

    args = []

    skip_console_logging_config = False

    log_setup = None

    # Set config option names
    loglevel_config_setting_name = "log_level"
    logfile_config_setting_name = "log_file"
    logfile_loglevel_config_setting_name = (
        "log_level_logfile"  # pylint: disable=invalid-name
    )

    @classmethod
    def setUpClass(cls):
        cls.root_dir = tempfile.mkdtemp(dir=RUNTIME_VARS.TMP)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.root_dir, ignore_errors=True)

    def setup_log(self):
        """
        Mock logger functions
        """
        testing_config = self.default_config.copy()
        testing_config["root_dir"] = self.root_dir
        for name in ("pki_dir", "cachedir"):
            testing_config[name] = name
        testing_config[self.logfile_config_setting_name] = getattr(
            self, self.logfile_config_setting_name, self.log_file
        )
        self.testing_config = testing_config
        self.addCleanup(setattr, self, "testing_config", None)
        self.log_setup = LogSetupMock()
        patcher = patch.multiple(
            log,
            setup_console_logger=self.log_setup.setup_console_logger,
            setup_extended_logging=self.log_setup.setup_extended_logging,
            setup_logfile_logger=self.log_setup.setup_logfile_logger,
            get_multiprocessing_logging_queue=self.log_setup.get_multiprocessing_logging_queue,
            setup_multiprocessing_logging_listener=self.log_setup.setup_multiprocessing_logging_listener,
            setup_temp_logger=self.log_setup.setup_temp_logger,
        )
        patcher.start()
        self.addCleanup(patcher.stop)
        self.addCleanup(setattr, self, "log_setup", None)

    # log level configuration tests

    def test_get_log_level_cli(self):
        """
        Tests that log level match command-line specified value
        """
        # Set defaults
        default_log_level = self.testing_config[self.loglevel_config_setting_name]

        # Set log level in CLI
        log_level = "critical"
        args = ["--log-level", log_level] + self.args

        parser = self.parser()
        with patch(self.config_func, MagicMock(return_value=self.testing_config)):
            parser.parse_args(args)
        with patch("salt.utils.parsers.is_writeable", MagicMock(return_value=True)):
            parser.setup_logfile_logger()

        console_log_level = getattr(parser.options, self.loglevel_config_setting_name)

        # Check console log level setting
        self.assertEqual(console_log_level, log_level)
        # Check console loggger log level
        self.assertEqual(self.log_setup.log_level, log_level)
        self.assertEqual(
            self.log_setup.config[self.loglevel_config_setting_name], log_level
        )
        self.assertEqual(self.log_setup.temp_log_level, log_level)
        # Check log file logger log level
        self.assertEqual(self.log_setup.log_level_logfile, default_log_level)

    def test_get_log_level_config(self):
        """
        Tests that log level match the configured value
        """
        args = self.args

        # Set log level in config
        log_level = "info"
        opts = self.testing_config.copy()
        opts.update({self.loglevel_config_setting_name: log_level})

        parser = self.parser()
        with patch(self.config_func, MagicMock(return_value=opts)):
            parser.parse_args(args)
        with patch("salt.utils.parsers.is_writeable", MagicMock(return_value=True)):
            parser.setup_logfile_logger()

        console_log_level = getattr(parser.options, self.loglevel_config_setting_name)

        # Check console log level setting
        self.assertEqual(console_log_level, log_level)
        # Check console loggger log level
        self.assertEqual(self.log_setup.log_level, log_level)
        self.assertEqual(
            self.log_setup.config[self.loglevel_config_setting_name], log_level
        )
        self.assertEqual(self.log_setup.temp_log_level, "error")
        # Check log file logger log level
        self.assertEqual(self.log_setup.log_level_logfile, log_level)

    def test_get_log_level_default(self):
        """
        Tests that log level match the default value
        """
        # Set defaults
        log_level = default_log_level = self.testing_config[
            self.loglevel_config_setting_name
        ]

        args = self.args

        parser = self.parser()
        with patch(self.config_func, MagicMock(return_value=self.testing_config)):
            parser.parse_args(args)
        with patch("salt.utils.parsers.is_writeable", MagicMock(return_value=True)):
            parser.setup_logfile_logger()

        console_log_level = getattr(parser.options, self.loglevel_config_setting_name)

        # Check log level setting
        self.assertEqual(console_log_level, log_level)
        # Check console loggger log level
        self.assertEqual(self.log_setup.log_level, log_level)
        # Check extended logger
        self.assertEqual(
            self.log_setup.config[self.loglevel_config_setting_name], log_level
        )
        self.assertEqual(self.log_setup.temp_log_level, "error")
        # Check log file logger
        self.assertEqual(self.log_setup.log_level_logfile, default_log_level)
        # Check help message
        self.assertIn(
            "Default: '{0}'.".format(default_log_level),
            parser.get_option("--log-level").help,
        )

    # log file configuration tests

    def test_get_log_file_cli(self):
        """
        Tests that log file match command-line specified value
        """
        # Set defaults
        log_level = self.testing_config[self.loglevel_config_setting_name]

        # Set log file in CLI
        log_file = "{0}_cli.log".format(self.log_file)
        args = ["--log-file", log_file] + self.args

        parser = self.parser()
        with patch(self.config_func, MagicMock(return_value=self.testing_config)):
            parser.parse_args(args)
        with patch("salt.utils.parsers.is_writeable", MagicMock(return_value=True)):
            parser.setup_logfile_logger()

        log_file_option = getattr(parser.options, self.logfile_config_setting_name)

        if not self.skip_console_logging_config:
            # Check console loggger
            self.assertEqual(self.log_setup.log_level, log_level)
            # Check extended logger
            self.assertEqual(
                self.log_setup.config[self.loglevel_config_setting_name], log_level
            )
            self.assertEqual(
                self.log_setup.config[self.logfile_config_setting_name], log_file
            )
        # Check temp logger
        self.assertEqual(self.log_setup.temp_log_level, "error")
        # Check log file setting
        self.assertEqual(log_file_option, log_file)
        # Check log file logger
        self.assertEqual(self.log_setup.log_file, log_file)

    def test_get_log_file_config(self):
        """
        Tests that log file match the configured value
        """
        # Set defaults
        log_level = self.testing_config[self.loglevel_config_setting_name]

        args = self.args

        # Set log file in config
        log_file = "{0}_config.log".format(self.log_file)
        opts = self.testing_config.copy()
        opts.update({self.logfile_config_setting_name: log_file})

        parser = self.parser()
        with patch(self.config_func, MagicMock(return_value=opts)):
            parser.parse_args(args)
        with patch("salt.utils.parsers.is_writeable", MagicMock(return_value=True)):
            parser.setup_logfile_logger()

        log_file_option = getattr(parser.options, self.logfile_config_setting_name)

        if not self.skip_console_logging_config:
            # Check console loggger
            self.assertEqual(self.log_setup.log_level, log_level)
            # Check extended logger
            self.assertEqual(
                self.log_setup.config[self.loglevel_config_setting_name], log_level
            )
            self.assertEqual(
                self.log_setup.config[self.logfile_config_setting_name], log_file
            )
        # Check temp logger
        self.assertEqual(self.log_setup.temp_log_level, "error")
        # Check log file setting
        self.assertEqual(log_file_option, log_file)
        # Check log file logger
        self.assertEqual(self.log_setup.log_file, log_file)

    def test_get_log_file_default(self):
        """
        Tests that log file match the default value
        """
        # Set defaults
        log_level = self.testing_config[self.loglevel_config_setting_name]
        log_file = self.testing_config[self.logfile_config_setting_name]
        default_log_file = self.default_config[self.logfile_config_setting_name]

        args = self.args

        parser = self.parser()
        with patch(self.config_func, MagicMock(return_value=self.testing_config)):
            parser.parse_args(args)
        with patch("salt.utils.parsers.is_writeable", MagicMock(return_value=True)):
            parser.setup_logfile_logger()

        log_file_option = getattr(parser.options, self.logfile_config_setting_name)

        if not self.skip_console_logging_config:
            # Check console loggger
            self.assertEqual(self.log_setup.log_level, log_level)
            # Check extended logger
            self.assertEqual(
                self.log_setup.config[self.loglevel_config_setting_name], log_level
            )
            self.assertEqual(
                self.log_setup.config[self.logfile_config_setting_name], log_file
            )
        # Check temp logger
        self.assertEqual(self.log_setup.temp_log_level, "error")
        # Check log file setting
        self.assertEqual(log_file_option, log_file)
        # Check log file logger
        self.assertEqual(self.log_setup.log_file, log_file)
        # Check help message
        self.assertIn(
            "Default: '{0}'.".format(default_log_file),
            parser.get_option("--log-file").help,
        )

    # log file log level configuration tests

    def test_get_log_file_level_cli(self):
        """
        Tests that file log level match command-line specified value
        """
        # Set defaults
        default_log_level = self.testing_config[self.loglevel_config_setting_name]

        # Set log file level in CLI
        log_level_logfile = "error"
        args = ["--log-file-level", log_level_logfile] + self.args

        parser = self.parser()
        with patch(self.config_func, MagicMock(return_value=self.testing_config)):
            parser.parse_args(args)
        with patch("salt.utils.parsers.is_writeable", MagicMock(return_value=True)):
            parser.setup_logfile_logger()

        log_level_logfile_option = getattr(
            parser.options, self.logfile_loglevel_config_setting_name
        )

        if not self.skip_console_logging_config:
            # Check console loggger
            self.assertEqual(self.log_setup.log_level, default_log_level)
            # Check extended logger
            self.assertEqual(
                self.log_setup.config[self.loglevel_config_setting_name],
                default_log_level,
            )
            self.assertEqual(
                self.log_setup.config[self.logfile_loglevel_config_setting_name],
                log_level_logfile,
            )
        # Check temp logger
        self.assertEqual(self.log_setup.temp_log_level, "error")
        # Check log file level setting
        self.assertEqual(log_level_logfile_option, log_level_logfile)
        # Check log file logger
        self.assertEqual(self.log_setup.log_level_logfile, log_level_logfile)

    def test_get_log_file_level_config(self):
        """
        Tests that log file level match the configured value
        """
        # Set defaults
        log_level = self.testing_config[self.loglevel_config_setting_name]

        args = self.args

        # Set log file level in config
        log_level_logfile = "info"
        opts = self.testing_config.copy()
        opts.update({self.logfile_loglevel_config_setting_name: log_level_logfile})

        parser = self.parser()
        with patch(self.config_func, MagicMock(return_value=opts)):
            parser.parse_args(args)
        with patch("salt.utils.parsers.is_writeable", MagicMock(return_value=True)):
            parser.setup_logfile_logger()

        log_level_logfile_option = getattr(
            parser.options, self.logfile_loglevel_config_setting_name
        )

        if not self.skip_console_logging_config:
            # Check console loggger
            self.assertEqual(self.log_setup.log_level, log_level)
            # Check extended logger
            self.assertEqual(
                self.log_setup.config[self.loglevel_config_setting_name], log_level
            )
            self.assertEqual(
                self.log_setup.config[self.logfile_loglevel_config_setting_name],
                log_level_logfile,
            )
        # Check temp logger
        self.assertEqual(self.log_setup.temp_log_level, "error")
        # Check log file level setting
        self.assertEqual(log_level_logfile_option, log_level_logfile)
        # Check log file logger
        self.assertEqual(self.log_setup.log_level_logfile, log_level_logfile)

    def test_get_log_file_level_default(self):
        """
        Tests that log file level match the default value
        """
        # Set defaults
        default_log_level = self.testing_config[self.loglevel_config_setting_name]

        log_level = default_log_level
        log_level_logfile = default_log_level

        args = self.args

        parser = self.parser()
        with patch(self.config_func, MagicMock(return_value=self.testing_config)):
            parser.parse_args(args)
        with patch("salt.utils.parsers.is_writeable", MagicMock(return_value=True)):
            parser.setup_logfile_logger()

        log_level_logfile_option = getattr(
            parser.options, self.logfile_loglevel_config_setting_name
        )

        if not self.skip_console_logging_config:
            # Check console loggger
            self.assertEqual(self.log_setup.log_level, log_level)
            # Check extended logger
            self.assertEqual(
                self.log_setup.config[self.loglevel_config_setting_name], log_level
            )
            self.assertEqual(
                self.log_setup.config[self.logfile_loglevel_config_setting_name],
                log_level_logfile,
            )
        # Check temp logger
        self.assertEqual(self.log_setup.temp_log_level, "error")
        # Check log file level setting
        self.assertEqual(log_level_logfile_option, log_level_logfile)
        # Check log file logger
        self.assertEqual(self.log_setup.log_level_logfile, log_level_logfile)
        # Check help message
        self.assertIn(
            "Default: '{0}'.".format(default_log_level),
            parser.get_option("--log-file-level").help,
        )

    def test_get_console_log_level_with_file_log_level(
        self,
    ):  # pylint: disable=invalid-name
        """
        Tests that both console log level and log file level setting are working together
        """
        log_level = "critical"
        log_level_logfile = "debug"

        args = ["--log-file-level", log_level_logfile] + self.args

        opts = self.testing_config.copy()
        opts.update({self.loglevel_config_setting_name: log_level})

        parser = self.parser()
        with patch(self.config_func, MagicMock(return_value=opts)):
            parser.parse_args(args)
        with patch("salt.utils.parsers.is_writeable", MagicMock(return_value=True)):
            parser.setup_logfile_logger()

        log_level_logfile_option = getattr(
            parser.options, self.logfile_loglevel_config_setting_name
        )

        if not self.skip_console_logging_config:
            # Check console loggger
            self.assertEqual(self.log_setup.log_level, log_level)
            # Check extended logger
            self.assertEqual(
                self.log_setup.config[self.loglevel_config_setting_name], log_level
            )
            self.assertEqual(
                self.log_setup.config[self.logfile_loglevel_config_setting_name],
                log_level_logfile,
            )
        # Check temp logger
        self.assertEqual(self.log_setup.temp_log_level, "error")
        # Check log file level setting
        self.assertEqual(log_level_logfile_option, log_level_logfile)
        # Check log file logger
        self.assertEqual(self.log_setup.log_level_logfile, log_level_logfile)

    @skipIf(salt.utils.platform.is_windows(), "Windows uses a logging listener")
    def test_log_created(self):
        """
        Tests that log file is created
        """
        args = self.args
        log_file = self.log_file
        log_file_name = self.logfile_config_setting_name
        opts = self.testing_config.copy()
        opts.update({"log_file": log_file})
        if log_file_name != "log_file":
            opts.update({log_file_name: getattr(self, log_file_name)})

        if log_file_name == "key_logfile":
            self.skipTest("salt-key creates log file outside of parse_args.")

        parser = self.parser()
        with patch(self.config_func, MagicMock(return_value=opts)):
            parser.parse_args(args)

        if log_file_name == "log_file":
            self.assertEqual(os.path.getsize(log_file), 0)
        else:
            self.assertEqual(os.path.getsize(getattr(self, log_file_name)), 0)

    def test_callbacks_uniqueness(self):
        """
        Test that the callbacks are only added once, no matter
        how many instances of the parser we create
        """
        mixin_container_names = (
            "_mixin_setup_funcs",
            "_mixin_process_funcs",
            "_mixin_after_parsed_funcs",
            "_mixin_before_exit_funcs",
        )
        parser = self.parser()
        nums_1 = {}
        for cb_container in mixin_container_names:
            obj = getattr(parser, cb_container)
            nums_1[cb_container] = len(obj)

        # The next time we instantiate the parser, the counts should be equal
        parser = self.parser()
        nums_2 = {}
        for cb_container in mixin_container_names:
            obj = getattr(parser, cb_container)
            nums_2[cb_container] = len(obj)
        self.assertDictEqual(nums_1, nums_2)


@skipIf(salt.utils.platform.is_windows(), "Windows uses a logging listener")
class MasterOptionParserTestCase(ParserBase, TestCase):
    """
    Tests parsing Salt Master options
    """

    def setUp(self):
        """
        Setting up
        """
        # Set defaults
        self.default_config = salt.config.DEFAULT_MASTER_OPTS.copy()
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/salt_master_parser_test"
        # Function to patch
        self.config_func = "salt.config.master_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.MasterOptionParser
        self.addCleanup(delattr, self, "parser")

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)


@skipIf(salt.utils.platform.is_windows(), "Windows uses a logging listener")
class MinionOptionParserTestCase(ParserBase, TestCase):
    """
    Tests parsing Salt Minion options
    """

    def setUp(self):
        """
        Setting up
        """
        # Set defaults
        self.default_config = salt.config.DEFAULT_MINION_OPTS.copy()
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/salt_minion_parser_test"
        # Function to patch
        self.config_func = "salt.config.minion_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.MinionOptionParser
        self.addCleanup(delattr, self, "parser")

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)


class ProxyMinionOptionParserTestCase(ParserBase, TestCase):
    """
    Tests parsing Salt Proxy Minion options
    """

    def setUp(self):
        """
        Setting up
        """
        # Set defaults
        self.default_config = salt.config.DEFAULT_MINION_OPTS.copy()
        self.default_config.update(salt.config.DEFAULT_PROXY_MINION_OPTS)
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/salt_proxy_minion_parser_test"
        # Function to patch
        self.config_func = "salt.config.proxy_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.ProxyMinionOptionParser
        self.addCleanup(delattr, self, "parser")

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)


@skipIf(salt.utils.platform.is_windows(), "Windows uses a logging listener")
class SyndicOptionParserTestCase(ParserBase, TestCase):
    """
    Tests parsing Salt Syndic options
    """

    def setUp(self):
        """
        Setting up
        """
        # Set config option names
        self.logfile_config_setting_name = "syndic_log_file"

        # Set defaults
        self.default_config = salt.config.DEFAULT_MASTER_OPTS.copy()
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/salt_syndic_parser_test"
        self.syndic_log_file = "/tmp/salt_syndic_log"
        # Function to patch
        self.config_func = "salt.config.syndic_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.SyndicOptionParser
        self.addCleanup(delattr, self, "parser")

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)
        if os.path.exists(self.syndic_log_file):
            os.unlink(self.syndic_log_file)


class SaltCMDOptionParserTestCase(ParserBase, TestCase):
    """
    Tests parsing Salt CLI options
    """

    def setUp(self):
        """
        Setting up
        """
        # Set mandatory CLI options
        self.args = ["foo", "bar.baz"]

        # Set defaults
        self.default_config = salt.config.DEFAULT_MASTER_OPTS.copy()
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/salt_cmd_parser_test"
        # Function to patch
        self.config_func = "salt.config.client_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.SaltCMDOptionParser
        self.addCleanup(delattr, self, "parser")

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)


class SaltCPOptionParserTestCase(ParserBase, TestCase):
    """
    Tests parsing salt-cp options
    """

    def setUp(self):
        """
        Setting up
        """
        # Set mandatory CLI options
        self.args = ["foo", "bar", "baz"]

        # Set defaults
        self.default_config = salt.config.DEFAULT_MASTER_OPTS.copy()
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/salt_cp_parser_test"
        # Function to patch
        self.config_func = "salt.config.master_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.SaltCPOptionParser
        self.addCleanup(delattr, self, "parser")

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)


class SaltKeyOptionParserTestCase(ParserBase, TestCase):
    """
    Tests parsing salt-key options
    """

    def setUp(self):
        """
        Setting up
        """
        self.skip_console_logging_config = True

        # Set config option names
        self.logfile_config_setting_name = "key_logfile"

        # Set defaults
        self.default_config = salt.config.DEFAULT_MASTER_OPTS.copy()
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/salt_key_parser_test"
        self.key_logfile = "/tmp/key_logfile"
        # Function to patch
        self.config_func = "salt.config.client_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.SaltKeyOptionParser
        self.addCleanup(delattr, self, "parser")

    # log level configuration tests

    def test_get_log_level_cli(self):
        """
        Tests that console log level option is not recognized
        """
        # No console log level will be actually set
        log_level = default_log_level = None

        option = "--log-level"
        args = self.args + [option, "error"]

        parser = self.parser()
        mock_err = ErrorMock()

        with patch("salt.utils.parsers.OptionParser.error", mock_err.error):
            parser.parse_args(args)

        # Check error msg
        self.assertEqual(mock_err.msg, "no such option: {0}".format(option))
        # Check console loggger has not been set
        self.assertEqual(self.log_setup.log_level, log_level)
        self.assertNotIn(self.loglevel_config_setting_name, self.log_setup.config)
        # Check temp logger
        self.assertEqual(self.log_setup.temp_log_level, "error")
        # Check log file logger log level
        self.assertEqual(self.log_setup.log_level_logfile, default_log_level)

    def test_get_log_level_config(self):
        """
        Tests that log level set in config is ignored
        """
        log_level = "info"
        args = self.args

        # Set log level in config and set additional mocked opts keys
        opts = {
            self.loglevel_config_setting_name: log_level,
            self.logfile_config_setting_name: "key_logfile",
            "log_fmt_logfile": None,
            "log_datefmt_logfile": None,
            "log_rotate_max_bytes": None,
            "log_rotate_backup_count": None,
        }

        parser = self.parser()
        with patch(self.config_func, MagicMock(return_value=opts)):
            parser.parse_args(args)
            with patch("salt.utils.parsers.is_writeable", MagicMock(return_value=True)):
                parser.setup_logfile_logger()

        # Check config name absence in options
        self.assertNotIn(self.loglevel_config_setting_name, parser.options.__dict__)
        # Check console loggger has not been set
        self.assertEqual(self.log_setup.log_level, None)
        self.assertNotIn(self.loglevel_config_setting_name, self.log_setup.config)
        # Check temp logger
        self.assertEqual(self.log_setup.temp_log_level, "error")
        # Check log file logger log level
        self.assertEqual(self.log_setup.log_level_logfile, log_level)

    def test_get_log_level_default(self):
        """
        Tests that log level default value is ignored
        """
        # Set defaults
        default_log_level = self.testing_config[self.loglevel_config_setting_name]

        log_level = None
        args = self.args

        parser = self.parser()
        parser.parse_args(args)

        with patch("salt.utils.parsers.is_writeable", MagicMock(return_value=True)):
            parser.setup_logfile_logger()

        # Check config name absence in options
        self.assertNotIn(self.loglevel_config_setting_name, parser.options.__dict__)
        # Check console loggger has not been set
        self.assertEqual(self.log_setup.log_level, log_level)
        self.assertNotIn(self.loglevel_config_setting_name, self.log_setup.config)
        # Check temp logger
        self.assertEqual(self.log_setup.temp_log_level, "error")
        # Check log file logger log level
        self.assertEqual(self.log_setup.log_level_logfile, default_log_level)

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)
        if os.path.exists(self.key_logfile):
            os.unlink(self.key_logfile)


class SaltCallOptionParserTestCase(ParserBase, TestCase):
    """
    Tests parsing Salt Minion options
    """

    def setUp(self):
        """
        Setting up
        """
        # Set mandatory CLI options
        self.args = ["foo.bar"]

        # Set defaults
        self.default_config = salt.config.DEFAULT_MINION_OPTS.copy()
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/salt_call_parser_test"
        # Function to patch
        self.config_func = "salt.config.minion_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.SaltCallOptionParser
        self.addCleanup(delattr, self, "parser")

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)


class SaltRunOptionParserTestCase(ParserBase, TestCase):
    """
    Tests parsing Salt Master options
    """

    def setUp(self):
        """
        Setting up
        """
        # Set mandatory CLI options
        self.args = ["foo.bar"]

        # Set defaults
        self.default_config = salt.config.DEFAULT_MASTER_OPTS.copy()
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/salt_run_parser_test"
        # Function to patch
        self.config_func = "salt.config.master_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.SaltRunOptionParser
        self.addCleanup(delattr, self, "parser")

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)


class SaltSSHOptionParserTestCase(ParserBase, TestCase):
    """
    Tests parsing Salt Master options
    """

    def setUp(self):
        """
        Setting up
        """
        # Set mandatory CLI options
        self.args = ["foo", "bar.baz"]

        # Set config option names
        self.logfile_config_setting_name = "ssh_log_file"

        # Set defaults
        self.default_config = salt.config.DEFAULT_MASTER_OPTS.copy()
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/salt_ssh_parser_test"
        self.ssh_log_file = "/tmp/ssh_logfile"
        # Function to patch
        self.config_func = "salt.config.master_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.SaltSSHOptionParser
        self.addCleanup(delattr, self, "parser")

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)
        if os.path.exists(self.ssh_log_file):
            os.unlink(self.ssh_log_file)


class SaltCloudParserTestCase(ParserBase, TestCase):
    """
    Tests parsing Salt Cloud options
    """

    def setUp(self):
        """
        Setting up
        """
        # Set mandatory CLI options
        self.args = ["-p", "foo", "bar"]

        # Set default configs
        # Cloud configs are merged with master configs in
        # config/__init__.py, so we'll do that here as well
        # As we need the 'user' key later on.
        self.default_config = salt.config.DEFAULT_MASTER_OPTS.copy()
        self.default_config.update(salt.config.DEFAULT_CLOUD_OPTS)
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/salt_cloud_parser_test"
        # Function to patch
        self.config_func = "salt.config.cloud_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.SaltCloudParser
        self.addCleanup(delattr, self, "parser")

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)


class SPMParserTestCase(ParserBase, TestCase):
    """
    Tests parsing Salt Cloud options
    """

    def setUp(self):
        """
        Setting up
        """
        # Set mandatory CLI options
        self.args = ["foo", "bar"]

        # Set config option names
        self.logfile_config_setting_name = "spm_logfile"

        # Set defaults
        self.default_config = salt.config.DEFAULT_MASTER_OPTS.copy()
        self.default_config.update(salt.config.DEFAULT_SPM_OPTS)
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/spm_parser_test"
        self.spm_logfile = "/tmp/spm_logfile"
        # Function to patch
        self.config_func = "salt.config.spm_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.SPMParser
        self.addCleanup(delattr, self, "parser")

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)
        if os.path.exists(self.spm_logfile):
            os.unlink(self.spm_logfile)


class SaltAPIParserTestCase(ParserBase, TestCase):
    """
    Tests parsing Salt Cloud options
    """

    def setUp(self):
        """
        Setting up
        """
        # Set mandatory CLI options
        self.args = []

        # Set config option names
        self.logfile_config_setting_name = "api_logfile"

        # Set defaults
        self.default_config = salt.config.DEFAULT_MASTER_OPTS.copy()
        self.default_config.update(salt.config.DEFAULT_API_OPTS)
        self.addCleanup(delattr, self, "default_config")

        # Log file
        self.log_file = "/tmp/salt_api_parser_test"
        self.api_logfile = "/tmp/api_logfile"
        # Function to patch
        self.config_func = "salt.config.api_config"

        # Mock log setup
        self.setup_log()

        # Assign parser
        self.parser = salt.utils.parsers.SaltAPIParser
        self.addCleanup(delattr, self, "parser")

    def tearDown(self):
        if os.path.exists(self.log_file):
            os.unlink(self.log_file)
        if os.path.exists(self.api_logfile):
            os.unlink(self.api_logfile)


class DaemonMixInTestCase(TestCase):
    """
    Tests the PIDfile deletion in the DaemonMixIn.
    """

    def setUp(self):
        """
        Setting up
        """
        # Setup mixin
        self.daemon_mixin = salt.utils.parsers.DaemonMixIn()
        self.daemon_mixin.config = {}
        self.daemon_mixin.config["pidfile"] = "/some/fake.pid"

    def tearDown(self):
        """
        Tear down test
        :return:
        """
        del self.daemon_mixin

    @patch("os.unlink", MagicMock())
    @patch("os.path.isfile", MagicMock(return_value=True))
    @patch("salt.utils.parsers.logger", MagicMock())
    def test_pid_file_deletion(self):
        """
        PIDfile deletion without exception.
        """
        self.daemon_mixin._mixin_before_exit()
        assert salt.utils.parsers.os.unlink.call_count == 1
        salt.utils.parsers.logger.info.assert_not_called()
        salt.utils.parsers.logger.debug.assert_not_called()

    @patch("os.unlink", MagicMock(side_effect=OSError()))
    @patch("os.path.isfile", MagicMock(return_value=True))
    @patch("salt.utils.parsers.logger", MagicMock())
    def test_pid_deleted_oserror_as_root(self):
        """
        PIDfile deletion with exception, running as root.
        """
        if salt.utils.platform.is_windows():
            patch_args = (
                "salt.utils.win_functions.is_admin",
                MagicMock(return_value=True),
            )
        else:
            patch_args = ("os.getuid", MagicMock(return_value=0))

        with patch(*patch_args):
            self.daemon_mixin._mixin_before_exit()
            assert salt.utils.parsers.os.unlink.call_count == 1
            salt.utils.parsers.logger.info.assert_called_with(
                "PIDfile could not be deleted: %s",
                format(self.daemon_mixin.config["pidfile"]),
            )
            salt.utils.parsers.logger.debug.assert_called()

    @patch("os.unlink", MagicMock(side_effect=OSError()))
    @patch("os.path.isfile", MagicMock(return_value=True))
    @patch("salt.utils.parsers.logger", MagicMock())
    def test_pid_deleted_oserror_as_non_root(self):
        """
        PIDfile deletion with exception, running as non-root.
        """
        if salt.utils.platform.is_windows():
            patch_args = (
                "salt.utils.win_functions.is_admin",
                MagicMock(return_value=False),
            )
        else:
            patch_args = ("os.getuid", MagicMock(return_value=1000))

        with patch(*patch_args):
            self.daemon_mixin._mixin_before_exit()
            assert salt.utils.parsers.os.unlink.call_count == 1
            salt.utils.parsers.logger.info.assert_not_called()
            salt.utils.parsers.logger.debug.assert_not_called()



# coding: utf-8

# Python libs
from __future__ import absolute_import

import logging

# Salt libs
import salt.beacons.log_beacon as log_beacon
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import mock_open, patch

# Salt testing libs
from tests.support.unit import TestCase

log = logging.getLogger(__name__)


_STUB_LOG_ENTRY = (
    "Jun 29 12:58:51 hostname sshd[6536]: "
    "pam_unix(sshd:session): session opened "
    "for user username by (uid=0)\n"
)


class LogBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.log
    """

    def setup_loader_modules(self):
        return {log_beacon: {"__context__": {"log.loc": 2}, "__salt__": {}}}

    def test_non_list_config(self):
        config = {}

        ret = log_beacon.validate(config)

        self.assertEqual(ret, (False, "Configuration for log beacon must be a list."))

    def test_empty_config(self):
        config = [{}]

        ret = log_beacon.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for log beacon must contain file option.")
        )

    def test_log_match(self):
        with patch("salt.utils.files.fopen", mock_open(read_data=_STUB_LOG_ENTRY)):
            config = [
                {"file": "/var/log/auth.log", "tags": {"sshd": {"regex": ".*sshd.*"}}}
            ]

            ret = log_beacon.validate(config)

            self.assertEqual(ret, (True, "Valid beacon configuration"))

            _expected_return = [
                {
                    "error": "",
                    "match": "yes",
                    "raw": _STUB_LOG_ENTRY.rstrip("\n"),
                    "tag": "sshd",
                }
            ]
            ret = log_beacon.beacon(config)
            self.assertEqual(ret, _expected_return)
